/*
 *  Shinobi Library
 *  Copyright (c) 1998 SEGA
 *
 *       Keyboard
 *
 *  Module  : Library Header File
 *  File    : sg_pdkbd.h
 *  Create  : 1998-03-25
 *  Modify  : 1999-01-20
 *  Version : 1.06
 *  Note    :
 */

#ifndef _SG_PDKBD_H_
#define _SG_PDKBD_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <sg_xpt.h>
#include <sg_pad.h>


/*
 *  DEFINES / MACROS
 */

/******** L[rbgATC ********************************************/
/******** Special keys bit assign *******************************************/
#define PDD_KEY_CTRL_RGUI     (1 << 7)     /* R-GUI                         */
#define PDD_KEY_CTRL_RALT     (1 << 6)     /* R-ALT                         */
#define PDD_KEY_CTRL_RSHIFT   (1 << 5)     /* R-SHIFT                       */
#define PDD_KEY_CTRL_RCTRL    (1 << 4)     /* R-CTRL                        */
#define PDD_KEY_CTRL_LGUI     (1 << 3)     /* L-GUI                         */
#define PDD_KEY_CTRL_LALT     (1 << 2)     /* L-ALT                         */
#define PDD_KEY_CTRL_LSHIFT   (1 << 1)     /* L-SHIFT                       */
#define PDD_KEY_CTRL_LCTRL    (1 << 0)     /* L-CTRL                        */

/******** LED_ *******************************************************/
/******** LED lighting state ************************************************/
#define PDD_LED_SHIFT         (1 << 7)     /* Shift                         */
#define PDD_LED_POWER         (1 << 6)     /* Power                         */
#define PDD_LED_KANA          (1 << 5)     /* Kana                          */
#define PDD_LED_SCRLOCK       (1 << 2)     /* Scroll Lock                   */
#define PDD_LED_CAPLOCK       (1 << 1)     /* Caps Lock                     */
#define PDD_LED_NUMLOCK       (1 << 0)     /* Num Lock                      */


/*
 *  TYPEDEFS
 */

/******** L[{[hf[^\ *******************************************/
/******** Keyboard data structure ******************************************/
typedef struct {
	Uint8 ctrl;                           /* L[̏                */
	                                      /* Special keys state            */
	Uint8 led;                            /* LED̓_                 */
	                                      /* LED lighting state            */
	Uint8 key[6];                         /* ĂL[R[h        */
	                                      /* key codes                     */
} PDS_KEYBOARD;

#if 0
/******** L[{[h\ *********************************************/
typedef struct {
	Uint8 lang;                           /* L[{[h                */
	Uint8 type;                           /* L[{[h^Cv              */
	Uint8 reserved[2];                    /* \                          */
} PDS_KEYBOARDINFO;

typedef struct {
	Uint8 used;
	Uint8 rate;
	Uint8 delay;
	Uint8 rate_count;
	Uint8 delay_count;
	Uint8 repeat_flag;
	Uint8 predata;
	Uint8 old_data[8];
	Uint32 port;
	Uint32 putp;
	Uint32 getp;
	Uint8* buf;
	Uint32 bufsize;
} PDS_KBDSTRUCT;

typedef PDS_KBDSTRUCT* PD_KBD;

#endif


/*
 *  PROTOTYPES
 */

/******** ჌xL[{[h֐ *******************************************/
/******** Low level keyboard functions *************************************/
const PDS_KEYBOARD* pdKbdGetData(Uint32 port);

#if 0
const PDS_KEYBOARDINFO* pdKbdGetInfo(Uint32 port);
#endif

/******** xL[{[h֐ *******************************************/
/******** High level keyboard functions ************************************/
#if 0
void pdKbdInit(void);
void pdKbdSetState(PD_KBD hKbd, Uint8 rate, Uint8 delay);
void pdKbdGetState(PD_KBD hKbd, Uint8* rate, Uint8* delay);
Sint32 pdKbdIsHit(PD_KBD hKbd);
PD_KBD pdKbdCreate(Uint32 port);
void pdKbdDestroy(PD_KBD kbd);
char pdKbdGetc(PD_KBD hKbd);
void pdKbdExecServer(void);
void pdKbdExecServerOne(PD_KBD hKbd);
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _SG_PDKBD_H_ */
