/*
 *
 *		ADXT Sample No.3(Playing AFS file)
 *
 */

/***************************************************************************
 *      CN[ht@C
 *      Include file
 ***************************************************************************/
#include	"shinobi.h"
#include	"sg_sd.h"
#include	"cri_adxf.h"
#include	"cri_adxt.h"

/***************************************************************************
 *      萔}N
 *      MACRO CONSTANT
 ***************************************************************************/
/*	p[eBVhc	*/
/*	Partition ID		*/
#define	PATID		(0)

/*	p[eBV̍őt@C			*/
/*	The number of maximum files in partition	*/
#define	MAX_NFILES	(256)

/*	AFSt@C (p[eBV)	*/
/*	Name of AFS file(partition)		*/
#define AFSFNAME	"PAT01.AFS"

/*	ő`l				*/
/*	The number of maximum channels	*/
#define	MAX_CH			(2)

/*	ĐőCDXg[									*/
/*	The number of maximum CD streams playing at the same time	*/
#define	MAX_CDSTM		(4)

/*	ōTvOg		*/
/*	Maximum sampling frequency	*/
#define	MAX_SFREQ		(44100)

/*	őADXTnh						*/
/*	The number of maximum ADXT handles		*/
#define	MAX_ADXT		(4)

/*	t@CĐp[N̈̑傫	*/
/*	Work area size of CD stream play	*/
#define	WKSIZE		(ADXT_CALC_WORK(MAX_CH, 1, MAX_CDSTM, MAX_SFREQ))

/***************************************************************************
 *      ϐ`
 *		Variable Declaration
 ***************************************************************************/
/*	p[eBV̈		*/
/*	Partition infomation area	*/
static char ptinfo[ADXF_CALC_PTINFO_SIZE(MAX_NFILES)];

/*	t@CĐp ADXT [N̈	*/
/*	Work area						*/
char work[MAX_ADXT][WKSIZE];

/***************************************************************************
 *      ֐`
 *      Function Declaration
 ***************************************************************************/

/***
 *		TEh̏
 *		Initialize sound section
 ***/

/*	TEhhCo̓]E					*/
/*	Initialize sound library and load sound driver	*/
void	sound_init(void *ptr)
{
	SDMEMBLK	memblk = NULL;

	/*	Initialize Sound library	 */
	sdLibInit(NULL, 0, 0);		/*	This api's argument is dummy now.  */

	/* 	Download and Initialize Sound driver.	 */
	/*	Create Memory block handle  */
	sdMemBlkCreate(&memblk);
	/*  Setting Memoby block handle  */
	sdMemBlkSetPrm(memblk, ptr, 0, SDD_MEMBLK_SYNC_FUNC, NULL);
	/*	Initialize Sound driver  */
	sdDrvInit(memblk);
	sdMemBlkDestroy(memblk);
}

/***
 *			[eBeB֐
 *			Utility function
 ***/

/*	f[^̓ǂݍ	*/
/*	Load data to memory	*/
void *load_file(char *fname, long *len)
{
	GDFS gdfs;
	long fsize, nsct;
	void *dat=NULL;

	if ( (gdfs=gdFsOpen(fname, NULL)) == NULL ) {
		for (;;) {
			njPrint(NJM_LOCATION(3, 5), "can't open %s", fname);
			njWaitVSync();
		}
	}
	gdFsGetFileSize(gdfs, &fsize);
	nsct = (fsize+2047)/2048;
	if ( (dat=syMalloc(nsct*2048)) != NULL ) {
		gdFsRead(gdfs, nsct, dat);
	}
	gdFsClose(gdfs);
	if ( len != NULL )
		*len = fsize;

	return dat;
}

/*	Ԃւ̕ϊ															*/
/*	Convert form sampling frequency and counter to time(hour,min,sec,frame)	*/
void sfcnt2time(long sf, long ncnt, long *hh, long *mm, long *ss, long *ff)
{
	register long nf = (long)((double)ncnt/(double)sf*100.0);

	*hh = nf/(60*60*100);
	*mm = nf/(60*100) - *hh*60;
	*ss = nf/100 - *hh*60*60 - *mm*60;
	*ff = nf%100;
}

/***
 *			TvvO
 *			Sample program
 ***/

/*	Ԃ̕\				*/
/*	Display status of program	*/
void adxtst_disp_stat(ADXT *adxt, long nadxt)
{
	char *ststr[6] =
		{"STOP   ", "DECINFO   ", "PREP   ", "PLAYING", "DECEND ", "PLAYEND"};
	long i, stat, sfreq, cnt;
	long hh, mm, ss, ff;

		njPrint(NJM_LOCATION(3, 3), "ADX Sample Program No.3");
		njPrint(NJM_LOCATION(3, 4), "   ( AFS File Stream Play )");
	for (i=0; i<nadxt; i++) {
		stat = ADXT_GetStat(adxt[i]);
		njPrint(NJM_LOCATION(3, 6+i*3), "ADXT %2d: Stat=%s", i, ststr[stat]);

		ADXT_GetTime(adxt[i], &cnt, &sfreq);
		sfcnt2time(sfreq,  cnt, &hh, &mm, &ss, &ff);
		njPrint(NJM_LOCATION(12, 7+i*3), "Time = %2d:%2d:%2d.%2d", 
														hh, mm, ss, ff);
	}
}

/*	ADX t@CĐTv	*/
/*	Playing ADX file			*/
void adxsmp(void)
{
	const NJS_PERIPHERAL* per;
	ADXT adxt[MAX_ADXT];
	long i;

	/*	AFS t@CĐpnh̐		*/
	/*	Create ADXT handle for playing AFS file	*/
	for (i=0; i<MAX_ADXT; i++) {
		if ( (adxt[i]=ADXT_Create(2, work[i], WKSIZE)) == NULL ) {
			for (;;) {
				njPrint(NJM_LOCATION(1, 27), "can't create ADXT\n");
				njWaitVSync();
			}
		}
	}

	for (;;) {
		per = njGetPeripheral(NJD_PORT_A0);

		/*	AFS t@CĐ		*/
		/*	Start to play AFS file	*/
		if (per->press & NJD_DGT_TA)
			ADXT_StartAfs(adxt[0], PATID, 0);
		if (per->press & NJD_DGT_TB)
			ADXT_StartAfs(adxt[0], PATID, 1);
		if (per->press & NJD_DGT_TX)
			ADXT_StartAfs(adxt[1], PATID, 2);
		if (per->press & NJD_DGT_TY)
			ADXT_StartAfs(adxt[1], PATID, 3);
		if (per->press & NJD_DGT_KR)
			ADXT_StartAfs(adxt[2], PATID, 4);
		if (per->press & NJD_DGT_KL)
			ADXT_StartAfs(adxt[2], PATID, 5);
		if (per->press & NJD_DGT_KD)
			ADXT_StartAfs(adxt[3], PATID, 6);
		if (per->press & NJD_DGT_KU)
			ADXT_StartAfs(adxt[3], PATID, 7);
		if (per->press & NJD_DGT_ST)
			for (i=0; i<MAX_ADXT; i++) ADXT_Stop(adxt[i]);

		adxtst_disp_stat(adxt, MAX_ADXT);
		njWaitVSync();
	}

	/*	ADXT nh̏	*/
	/*	Destroy ADXT handle	*/
	for (i=0; i<MAX_ADXT; i++)
		ADXT_Destroy(adxt[i]);
}

/***
 *		C
 *		MAIN
 ***/

/*	AvP[VI	*/
/*	Finalize application		*/
void usr_exit(void)
{
	/*	ADXTCuI				*/
	/*	Finalize ADXT library				*/
	ADXT_Finish();
	/*	ECȕI				*/
	/*	Finalize SHINOBI					*/
	sbExitSystem();
	/*	VvvCփWv			*/
	/*	Jump to Simple player				*/
	syBtExit();
}

/*	GDt@CVXẽG[ɋN֐			*/
/*	Callback function when an error occur in GD file system	*/
void usr_gd_err_func(void *obj, Sint32 errcode)
{
	if (errcode == GDD_ERR_TRAYOPEND || errcode == GDD_ERR_UNITATTENT) {
		/*	AvP[VI	*/
		/*	Finalize application		*/
		usr_exit();
 	}
 	return;
}

/*	[UV-SYNC荞݂ɓo^֐	*/
/*	Callback function when V-sync interrupt	*/
void usr_vsync_func(void)
{
	Sint32 dstat;

    /*	TEhCũT[o֐		*/
	/*	Server function of sound library	*/
    sdSysServer();

	/*	GDhCȕԃ`FbN	*/
	/*	check status of GD-drive	*/
	dstat = gdFsGetDrvStat();
	if (dstat == GDD_DRVSTAT_OPEN || dstat == GDD_DRVSTAT_BUSY) {
		gdFsReqDrvStat();
	}
	return;
}

/*	ADXCũG[ɋN֐				*/
/*	Callback function when an error occur in ADX library	*/
void usr_adx_err_func(void *obj, char *msg)
{
	/*	
	 *	ADXŁAG[Ƃ̊֐Ăяo܂B
	 *	msg ɃG[bZ[Wn܂B
	 *	msǵAR5WX^Ɋi[Ă̂ŁAR5WX^̃AhX
	 *	_v邱ƂɂăG[bZ[Wm邱Ƃł܂B
	 *	̊֐́AV-Sync荞݂Ăяo邱Ƃ܂̂ŁA
	 *	Ninjå֐͎gpȂłB
	 *	܂AAvP[V[XƂ́Aɖ߂悤
 	 *	֐ɂĂB
	 *	This function call when error occur in ADX library.
	 *	Error message is put into 'msg' variable.
	 *	'msg' house R5 register. You can look error message in address of 
	 *	R5 register.
	 *	You cannot call Ninja function because this function is called 
	 *	from V-Int.
	 *	You must change this to no operation function when you release 
	 *	the application.
	 */

	for (;;);		/*	[Xɂ͖					*/
					/*	Delete when you release application.	*/

}


void main(void)
{
	void *snddrv;

	/*	ECȕ	*/
	/*	Initialize SHINOBI		*/
    njSetBorderColor( 0x00000000 ) ;
	sbInitSystem(NJD_RESOLUTION_320x240_NTSCNI,NJD_FRAMEBUFFER_MODE_RGB565,1);
	njInitVertexBuffer( -30000, 0, 30000, 0, 0);
    njInitPrint(NULL, 0, 0);
	/*	GDt@CVXeG[R[obN֐̓o^	*/
	/*	Entry callback function of GD file system error	*/
	gdFsEntryErrFuncAll((void *)usr_gd_err_func, NULL);

	/*	TEhCȕ	*/
	/*	Prepare sound library		*/
	snddrv = load_file("manatee.drv", NULL);
	sound_init(snddrv);
	syFree(snddrv);
	/*	DMA ][h				*/
	/*	Set transfer mode to DMA  	*/
	sdMemBlkSetTransferMode(SDE_MEMBLK_TRANSFER_MODE_DMA);

	njSetVSyncFunction((void (*)(void))usr_vsync_func);

	/*	ADXCu		*/
	/*	Initialize ADX library	*/
	ADXT_Init();
	/*	G[R[obN֐̓o^		*/
	/*	Entry ADX error callback function	*/
	ADXT_EntryErrFunc(usr_adx_err_func, NULL);
	/*	p[eBV̓ǂݍ	*/
	/*	Load partition infomation		*/
	ADXF_LoadPartition(PATID, AFSFNAME, ptinfo, MAX_NFILES);

	for (;;)
		adxsmp();					/*	ADX t@CĐTv	*/
									/*	Playing ADX file			*/

	/*	AvP[VI	*/
	/*	Finalize application		*/
	usr_exit();
}


/* End of file */
