/*
 *
 *		ADXT Sample No.4(ADX file system)
 *
 */

/***************************************************************************
 *      CN[ht@C
 *      Include file
 ***************************************************************************/
#include	"shinobi.h"
#include	"sg_sd.h"
#include	"cri_adxf.h"
#include	"cri_adxt.h"

#include	"pat02.h"

/***************************************************************************
 *      萔}N
 *      MACRO CONSTANT
 ***************************************************************************/
/*	p[eBVhc	*/
/*	Partition ID		*/
#define	PATID		(0)

/*	p[eBV̍őt@C			*/
/*	The number of maximum files in partition	*/
#define	MAX_NFILES	(256)

/*	AFSt@C (p[eBV)	*/
/*	Name of AFS file(partition)		*/
#define AFSFNAME	"PAT02.AFS"

/*	ő`l				*/
/*	The number of maximum channels	*/
#define	MAX_CH			(2)

/*	ĐőCDXg[									*/
/*	The number of maximum CD streams playing at the same time	*/
#define	MAX_CDSTM		(1)

/*	ōTvOg		*/
/*	Maximum sampling frequency	*/
#define	MAX_SFREQ		(44100)

/*	őADXFnh						*/
/*	The number of maximum ADXF handles		*/
#define	MAX_ADXF		(5)

/*	t@CĐp[N̈̑傫	*/
/*	Work area size of CD stream play	*/
#define	WKSIZE		(ADXT_CALC_WORK(MAX_CH, 1, MAX_CDSTM, MAX_SFREQ))

/***************************************************************************
 *      ϐ`
 *		Variable Declaration
 ***************************************************************************/

/*	p[eBV̈		*/
/*	Partition infomation area	*/
static char ptinfo[ADXF_CALC_PTINFO_SIZE(MAX_NFILES)];

/*	t@CĐp ADXT [N̈	*/
/*	Work area						*/
char work[WKSIZE];

/*	f[^ǂݍݗ̈		*/
/*	Area for loading data	*/
char *data_buf;

/***************************************************************************
 *      ֐`
 *      Function Declaration
 ***************************************************************************/

/***
 *		TEh̏
 *		Initialize sound section
 ***/

/*	TEhhCo̓]E					*/
/*	Initialize sound library and load sound driver	*/
void	sound_init(void *ptr)
{
	SDMEMBLK	memblk = NULL;

	/*	Initialize Sound library	 */
	sdLibInit(NULL, 0, 0);		/*	This api's argument is dummy now.  */

	/* 	Download and Initialize Sound driver.	 */
	/*	Create Memory block handle  */
	sdMemBlkCreate(&memblk);
	/*  Setting Memoby block handle  */
	sdMemBlkSetPrm(memblk, ptr, 0, SDD_MEMBLK_SYNC_FUNC, NULL);
	/*	Initialize Sound driver  */
	sdDrvInit(memblk);
	sdMemBlkDestroy(memblk);
}

/***
 *			[eBeB֐
 *			Utility function
 ***/

/*	f[^̓ǂݍ	*/
/*	Load data to memory	*/
void *load_file(char *fname, long *len)
{
	GDFS gdfs;
	long fsize, nsct;
	void *dat=NULL;

	if ( (gdfs=gdFsOpen(fname, NULL)) == NULL ) {
		for (;;) {
			njPrint(NJM_LOCATION(3, 5), "can't open %s", fname);
			njWaitVSync();
		}
	}
	gdFsGetFileSize(gdfs, &fsize);
	nsct = (fsize+2047)/2048;
	if ( (dat=syMalloc(nsct*2048)) != NULL ) {
		gdFsRead(gdfs, nsct, dat);
	}
	gdFsClose(gdfs);
	if ( len != NULL )
		*len = fsize;

	return dat;
}

/*	Ԃւ̕ϊ															*/
/*	Convert form sampling frequency and counter to time(hour,min,sec,frame)	*/
void sfcnt2time(long sf, long ncnt, long *hh, long *mm, long *ss, long *ff)
{
	register long nf = (long)((double)ncnt/(double)sf*100.0);

	*hh = nf/(60*60*100);
	*mm = nf/(60*100) - *hh*60;
	*ss = nf/100 - *hh*60*60 - *mm*60;
	*ff = nf%100;
}

/***
 *			TvvO
 *			Sample program
 ***/

/*	Ԃ̕\				*/
/*	Display status of program	*/
void adxtst_disp_stat(ADXF *adxf, long nadxf)
{
	long i, j;
	char *ststr[5] = {"    ", "STOP", "READ", "END ", "ERR "};

	njPrint(NJM_LOCATION(3, 3), "ADX Sample Program No.4");
	njPrint(NJM_LOCATION(3, 4), "   ( ADX File system )");
	for (i=0; i<nadxf; i++) {
		njPrint(NJM_LOCATION(3+i*5, 6), "%s", ststr[ADXF_GetStat(adxf[i])]);
	}
	for (i=0; i<20; i++) {
		for (j=0; j<32; j++)
			njPrint(NJM_LOCATION(3+j, 8+i), "%c", data_buf[i*2048*5+j]);
	}
}

/*	f[^̈̃NA	*/
/*	Clear data area		*/
void adxtst_clear_data(void)
{
	memset(data_buf, 'x', 100*2048);
}

/*	ADXt@CĐAFSt@Cǂݍ݃Tv	*/
/*	Load AFS file during playing ADX file			*/
void adxsmp(void)
{
	const NJS_PERIPHERAL* per;
	long i;
	ADXF adxf[MAX_ADXF];
	ADXT adxt;

	/*	ADXt@CĐpnh̐		*/
	/*	Create ADXT handle					*/
	adxt = ADXT_Create(2, work, WKSIZE);
	/*	ĐJn		*/
	/*	Start to play	*/
	ADXT_StartFname(adxt, "loopsmp.adx");

	/*	AFSt@C̃I[v	*/
	/*	Open AFS file			*/
	adxf[0] = ADXF_OpenAfs(PATID, TEST1_BIN);
	adxf[1] = ADXF_OpenAfs(PATID, TEST2_BIN);
	adxf[2] = ADXF_OpenAfs(PATID, TEST3_BIN);
	adxf[3] = ADXF_OpenAfs(PATID, TEST4_BIN);
	adxf[4] = ADXF_OpenAfs(PATID, TEST5_BIN);

	for (;;) {
		per = njGetPeripheral(NJD_PORT_A0);

		/*	AFSt@C̓ǂݍ݊Jn	*/
		/*	Start to load AFS file		*/
		if (per->press & NJD_DGT_TA) {
			if ( ADXF_Tell(adxf[0]) ==  ADXF_GetFsizeSct(adxf[0]) )
				ADXF_Seek(adxf[0], 0, ADXF_SEEK_SET);
			ADXF_ReadNw32(adxf[0], 100, data_buf);
		}
		if (per->press & NJD_DGT_TB) {
			if ( ADXF_Tell(adxf[1]) ==  ADXF_GetFsizeSct(adxf[1]) )
				ADXF_Seek(adxf[1], 0, ADXF_SEEK_SET);
			ADXF_ReadNw32(adxf[1], 100, data_buf);
		}
		if (per->press & NJD_DGT_TX) {
			if ( ADXF_Tell(adxf[2]) ==  ADXF_GetFsizeSct(adxf[2]) )
				ADXF_Seek(adxf[2], 0, ADXF_SEEK_SET);
			ADXF_ReadNw32(adxf[2], 100, data_buf);
		}
		if (per->press & NJD_DGT_TY) {
			if ( ADXF_Tell(adxf[3]) ==  ADXF_GetFsizeSct(adxf[3]) )
				ADXF_Seek(adxf[3], 0, ADXF_SEEK_SET);
			ADXF_ReadNw32(adxf[3], 100, data_buf);
		}
		if (per->press & NJD_DGT_KL) {
			if ( ADXF_Tell(adxf[4]) ==  ADXF_GetFsizeSct(adxf[4]) )
				ADXF_Seek(adxf[4], 0, ADXF_SEEK_SET);
			ADXF_ReadNw32(adxf[4], 100, data_buf);
		}
		if (per->press & NJD_DGT_ST)
			adxtst_clear_data();

		adxtst_disp_stat(adxf, MAX_ADXF);
		njWaitVSync();
	}

	/*	AFSt@C̃N[Y	*/
	/*	Close AFS file			*/
	for (i=0; i<MAX_ADXF; i++)
		ADXF_Close(adxf[i]);

	/*	ADXT nh̏	*/
	/*	Destroy ADXT handle	*/
	ADXT_Destroy(adxt);
}

/***
 *		C
 *		MAIN
 ***/

/*	AvP[VI	*/
/*	Finalize application		*/
void usr_exit(void)
{
	/*	ADXTCuI				*/
	/*	Finalize ADXT library				*/
	ADXT_Finish();
	/*	ECȕI				*/
	/*	Finalize SHINOBI					*/
	sbExitSystem();
	/*	VvvCփWv			*/
	/*	Jump to Simple player				*/
	syBtExit();
}

/*	GDt@CVXẽG[ɋN֐			*/
/*	Callback function when an error occur in GD file system	*/
void usr_gd_err_func(void *obj, Sint32 errcode)
{
	if (errcode == GDD_ERR_TRAYOPEND || errcode == GDD_ERR_UNITATTENT) {
		/*	AvP[VI	*/
		/*	Finalize application		*/
		usr_exit();
 	}
 	return;
}

/*	[UV-SYNC荞݂ɓo^֐	*/
/*	Callback function when V-sync interrupt	*/
void usr_vsync_func(void)
{
	Sint32 dstat;

    /*	TEhCũT[o֐		*/
	/*	Server function of sound library	*/
    sdSysServer();

	/*	GDhCȕԃ`FbN	*/
	/*	check status of GD-drive	*/
	dstat = gdFsGetDrvStat();
	if (dstat == GDD_DRVSTAT_OPEN || dstat == GDD_DRVSTAT_BUSY) {
		gdFsReqDrvStat();
	}
	return;
}

/*	ADXCũG[ɋN֐				*/
/*	Callback function when an error occur in ADX library	*/
void usr_adx_err_func(void *obj, char *msg)
{
	/*	
	 *	ADXŁAG[Ƃ̊֐Ăяo܂B
	 *	msg ɃG[bZ[Wn܂B
	 *	msǵAR5WX^Ɋi[Ă̂ŁAR5WX^̃AhX
	 *	_v邱ƂɂăG[bZ[Wm邱Ƃł܂B
	 *	̊֐́AV-Sync荞݂Ăяo邱Ƃ܂̂ŁA
	 *	Ninjå֐͎gpȂłB
	 *	܂AAvP[V[XƂ́Aɖ߂悤
 	 *	֐ɂĂB
	 *	This function call when error occur in ADX library.
	 *	Error message is put into 'msg' variable.
	 *	'msg' house R5 register. You can look error message in address of 
	 *	R5 register.
	 *	You cannot call Ninja function because this function is called 
	 *	from V-Int.
	 *	You must change this to no operation function when you release 
	 *	the application.
	 */

	for (;;);		/*	[Xɂ͖					*/
					/*	Delete when you release application.	*/

}

void main(void)
{
	void *snddrv;

	/*	ECȕ	*/
	/*	Initialize SHINOBI		*/
    njSetBorderColor( 0x00000000 ) ;
	sbInitSystem(NJD_RESOLUTION_320x240_NTSCNI,NJD_FRAMEBUFFER_MODE_RGB565,1);
	njInitVertexBuffer( -30000, 0, 30000, 0, 0);
    njInitPrint(NULL, 0, 0);
	/*	GDt@CVXeG[R[obN֐̓o^	*/
	/*	Entry callback function of GD file system error	*/
	gdFsEntryErrFuncAll((void *)usr_gd_err_func, NULL);

	/*	TEhCȕ	*/
	/*	Prepare sound library		*/
	snddrv = load_file("manatee.drv", NULL);
	sound_init(snddrv);
	syFree(snddrv);
	/*	DMA ][h				*/
	/*	Set transfer mode to DMA  	*/
	sdMemBlkSetTransferMode(SDE_MEMBLK_TRANSFER_MODE_DMA);

	njSetVSyncFunction((void (*)(void))usr_vsync_func);

	/*	ADXCu		*/
	/*	Initialize ADX library	*/
	ADXT_Init();
	/*	G[R[obN֐̓o^		*/
	/*	Entry ADX error callback function	*/
	ADXT_EntryErrFunc(usr_adx_err_func, NULL);
	/*	p[eBV̓ǂݍ	*/
	/*	Load partition infomation		*/
	ADXF_LoadPartition(PATID, AFSFNAME, ptinfo, MAX_NFILES);

	data_buf = syMalloc(100*2048);
	adxtst_clear_data();


	for (;;)
		adxsmp();					/*	ADX t@CĐTv	*/
									/*	Playing ADX file			*/

	/*	AvP[VI	*/
	/*	Finalize application		*/
	usr_exit();
}


/* End of file */
