/*
 *
 *		Middleware TrueMotion Player Sample
 *
 */

/***************************************************************************
 *      CN[ht@C
 *      Include file
 ***************************************************************************/
#include <shinobi.h>
#include <sg_sd.h>
#include <sg_mw.h>


/***************************************************************************
 *      萔}N
 *      MACRO CONSTANT
 ***************************************************************************/

/*	\[h		*/
/*	Display mode	*/
#define SYS_MODE		NJD_RESOLUTION_640x480_NTSCI
#define SYS_FRAME		NJD_FRAMEBUFFER_MODE_ARGB8888
#define SYS_COUNT		1

/*	_obt@TCY(1}CiXɂ2V[eV[h)			*/
/*	Vertex buffer size(2V latency mode is when a valiable is minus.)	*/
#define VB_OP			100000
#define VB_OM			0
#define VB_TP			100000
#define VB_TM			0
#define VB_PT			100000

/*	NINJA			*/
#define	TEX_NUM			100
#define	TEXNAME_NUM		2
#define	TEX_BUFSIZE		0x20800

/*	G[bZ[W		*/
/*	Length of error message	*/
#define ERRMSG_LEN		256


/***************************************************************************
 *      }N
 *      Process MACRO
 ***************************************************************************/

/*	tOON/OFF̔]		*/
/*	Turn over flag(ON/OFF)	*/
#define NOT_FLAG(flag)		(((flag) == OFF) ? ON : OFF)


/***************************************************************************
 *      ϐ`
 *		Variable Declaration
 ***************************************************************************/

/*	G[		*/
/*	The number of error	*/
Sint32 errcnt = 0;

/*	G[bZ[W	*/
/*	Error message		*/
char errmsg[ERRMSG_LEN] = "(OK)";

/*	\tO				*/
/*	Flag for a show of character	*/
Sint32 disp_flag = ON;

/*	|[Y		*/
/*	Pause status	*/
Sint32 pause = OFF;

/*	NINJA				*/
NJS_TEXMEMLIST	tex[TEX_NUM];
NJS_TEXINFO		info[TEXNAME_NUM];
NJS_TEXNAME		texname[TEXNAME_NUM];
NJS_TEXLIST		texlist = {texname, TEXNAME_NUM};
Sint8			texbuf[TEX_BUFSIZE];
Uint16			framebuf[1024*512*2];

/***************************************************************************
 *      ֐`
 *      Function Declaration
 ***************************************************************************/
/***
 *		TEh̏
 *		Initialize sound section
 ***/

/*	TEhhCo̓]E					*/
/*	Initialize sound library and load sound driver	*/
void SoundInit(void *sndptr, void *mltptr)
{
	SDMEMBLK	memblk = NULL;

	/*	Initialize Sound library	 */
	sdLibInit(NULL, 0, 0);		/*	This api's argument is dummy now.  */

	/* 	Download and Initialize Sound driver.	 */
	/*	Create Memory block handle  */
	sdMemBlkCreate(&memblk);
	/*  Setting Memory block handle  */
	sdMemBlkSetPrm(memblk, sndptr, 0, SDD_MEMBLK_SYNC_FUNC, NULL);
	/*	Initialize Sound driver  */
	sdDrvInit(memblk);

	/*	Download and Initialize Multi unit	*/
	/*	Setting Memory block handle	*/
	sdMemBlkSetPrm(memblk, mltptr, 0, SDD_MEMBLK_SYNC_FUNC, NULL);
	sdDownloadMultiUnit(memblk);

	sdMemBlkDestroy(memblk);

	/*	Truemotion Only CPU mode	*/
#if	0
	/*	][h̐ݒ		*/
	/*	Setting transfer mode	*/
	sdMemBlkSetTransferMode(SDE_MEMBLK_TRANSFER_MODE_DMA);
#endif

	return;
}

/***
 *			[eBeB֐
 *			Utility function
 ***/

/*	f[^̓ǂݍ	*/
/*	Load data to memory	*/
void *LoadFile(char *fname, long *len)
{
	GDFS			gdfs;
	long			fsize, nsct;
	void			*dat;

	gdfs = gdFsOpen(fname, NULL);
	if (gdfs == NULL) {
		for (;;) ;
	}
	gdFsGetFileSize(gdfs, &fsize);
	nsct = (fsize + 2047) / 2048;
	dat = syMalloc(nsct * 2048);
	if (dat == NULL) {
		for (;;) ;
	}
	gdFsRead(gdfs, nsct, dat);
	gdFsClose(gdfs);
	if (len != NULL) {
		*len = fsize;
	}

	return dat;
}

/*	msec -> time(hour,min,sec,frame)	*/
void Msec2Time(long mstime, long *hh, long *mm, long *ss, long *ff)
{
	*hh = mstime / (60*60*1000);
	*mm = (mstime - *hh*60*60*1000) / (60*1000);
	*ss = (mstime - *hh*60*60*1000 - *mm*60*1000) / 1000;
	*ff = mstime % 1000;

	return;
}

/***
 *			~hEFA̍Đ
 *			Playing middleware
 ***/

/*	~hEFÃG[ɋN֐			*/
/*	Callback function when an error occur in middleware	*/
void UsrMwErrFunc(void *obj, char *ecode)
{
	errcnt++;
	strncpy(errmsg, ecode, ERRMSG_LEN);

	return;
}

/*	̕\			*/
/*	Display infomation	*/
void DispInfo(MWPLY ply, char *fname)
{
	long			time, tunit, hh, mm, ss, ff, time_ms;
	float			timef;

	mwPlyGetTime(ply, &time, &tunit);
	timef = ((float)time / (float)tunit) * 1000.0;
	time_ms = (long)timef;
	Msec2Time(time_ms, &hh, &mm, &ss, &ff);
	ff = ff*30/1000;
	njPrint(NJM_LOCATION(0, 3), "File: %s  Time:%02d:%02d:%02d.%02d", 
							fname, hh, mm, ss, ff);
	njPrint(NJM_LOCATION(0, 4), "Error%2d:%s", errcnt, errmsg);

	return;
}

/*	̍Đ					*/
/*	Playing the animation file	*/
void SmpPlay(char *fname)
{
	MWPLY					ply;
	MWS_PLY_CPRM_TM			cprm;
	MWE_PLY_STAT			stat;
	const PDS_PERIPHERAL	*per;

	/*	nh̐	*/
	/*	Create handle	*/
	ply = mwPlyCreateTM(&cprm);
	if (ply == NULL) {
		for (;;) ;
	}

	mwPlyEntryErrFunc(UsrMwErrFunc, &ply);
	mwPlyPause(ply, pause);				/*	OĐ̃|[YԂێ		*/
										/*	Keep the last time pause status	*/
	mwPlyStartFname(ply, fname);		/*	ĐJn						*/
										/*	Start to play middleware		*/
	while (1) {
		njWaitVSync();
		mwPlyStartFrame();
		per = pdGetPeripheral(NJD_PORT_A0);
		if (per->press & PDD_DGT_TA) {
			disp_flag = NOT_FLAG(disp_flag);
		}
		if (per->press & PDD_DGT_TX) {
			pause = ON;
			mwPlyPause(ply, ON);		/*	ꎞ~				*/
										/*	Pause					*/
		}
		if (per->press & PDD_DGT_TY) {
			pause = OFF;
			mwPlyPause(ply, OFF);		/*	ꎞ~			*/
										/*	Release pause status	*/
		}
		if (per->press & PDD_DGT_ST) {
			break;
		}
		stat = mwPlyGetStat(ply);
		if ((stat == MWE_PLY_STAT_PLAYEND) || (stat == MWE_PLY_STAT_ERROR)) {
			break;
		}
		if (disp_flag == ON) {
			DispInfo(ply, fname);
		}
		mwPlyExecServer();
	}
	mwPlyStop(ply);						/*	Đ~ 					*/
										/*	Stop to play middleware		*/
	njWaitVSync();						/*	pbh̏Ԏ擾̂		*/
										/*	for getting status of PAD	*/
	mwPlyDestroy(ply);					/*	nh				*/
										/*	Destroy handle				*/
	return;
}

/***
 *		C
 *		MAIN
 ***/

/*	AvP[VI	*/
/*	Finalize application		*/
void UsrExit(void)
{
	mwPlyFinishTM();		/*	CuI					*/
							/*	Finalize Middleware					*/
	sbExitSystem();			/*	ECȕI				*/
							/*	Finalize SHINOBI					*/
	syBtExit();				/*	VvvCփWv			*/
							/*	Jump to Simple player				*/
}

/*	GDt@CVXẽG[ɋN֐			*/
/*	Callback function when an error occur in GD file system	*/
void UsrGdErrFunc(void *obj, Sint32 errcode)
{
	if (errcode == GDD_ERR_TRAYOPEND || errcode == GDD_ERR_UNITATTENT) {
		UsrExit();					/*	AvP[VI	*/
									/*	Finalize application		*/
	}

	return;
}

/*	[UV-SYNC荞݂ɓo^֐	*/
/*	Callback function when V-sync interrupt	*/
void UsrVsyncFunc(void)
{
	Sint32 dstat;

    /*	TEhCũT[o֐		*/
	/*	Server function of sound library	*/
	sdSysServer();

	/*	hAI[v`FbN	*/
	/*	Check door open			*/
	dstat = gdFsGetDrvStat();
	if (dstat == GDD_DRVSTAT_OPEN || dstat == GDD_DRVSTAT_BUSY) {
		gdFsReqDrvStat();
	}

	return;
}

void main(void)
{
	void				*snddrv;
	void				*mltfile;

	/*	ECȕ	*/
	/*	Initialize SHINOBI		*/
	njSetBorderColor(0x00000000);
	sbInitSystem(SYS_MODE, SYS_FRAME, SYS_COUNT);
	njInitVertexBuffer(VB_OP, VB_OM, VB_TP, VB_TM, VB_PT);
	njInitPrint(NULL, 0, NJD_TEXFMT_ARGB_8888);
	njPrintSize(12);

	/*	GDt@CVXeG[R[obN֐̓o^	*/
	/*	Entry callback function of GD file system error	*/
	gdFsEntryErrFuncAll((void *)UsrGdErrFunc, NULL);

	/*	TEhCȕ	*/
	/*	Prepare sound library		*/
	snddrv  = LoadFile("manatee.drv", NULL);
	mltfile = LoadFile("stream.mlt", NULL);
	SoundInit(snddrv, mltfile);

	/*	NINJA			*/
	/*	Initialize NINJA	*/
	njInitTexture(tex, TEX_NUM);
	njInitTextureBuffer(texbuf, TEX_BUFSIZE);
	njSetTextureInfo(&info[0], framebuf,
					(NJD_TEXFMT_YUV_422 | NJD_TEXFMT_STRIDE), 1024, 512);
	njSetTextureInfo(&info[1], framebuf + 1024 * 512,
					(NJD_TEXFMT_YUV_422 | NJD_TEXFMT_STRIDE), 1024, 512);
	njSetTextureName(&texname[0], &info[0], 0,
					(NJD_TEXATTR_TYPE_MEMORY | NJD_TEXATTR_GLOBALINDEX));
	njSetTextureName(&texname[1], &info[1], 1,
					(NJD_TEXATTR_TYPE_MEMORY | NJD_TEXATTR_GLOBALINDEX));
	njLoadTexture(&texlist);
	njSetTexture(&texlist);

	njSetVSyncFunction(UsrVsyncFunc);

	/*	~hEFACu	*/
	/*	Initialize middleware			*/
	mwPlyInitTM();

	for (;;) {
		SmpPlay("SAMPLE.AVI");		/*	̍Đ			*/
									/*	Playing animation	*/
	}

	UsrExit();						/*	AvP[VI	*/
									/*	Finalize application		*/
}


/* end of file */
