/*
 *
 *		Middleware Wave Player Sample
 *
 */

/***************************************************************************
 *      CN[ht@C
 *      Include file
 ***************************************************************************/
#include	<shinobi.h>
#include	<sg_sd.h>
#include	<sg_mw.h>

/***************************************************************************
 *      ϐ`
 *		Variable Declaration
 ***************************************************************************/

/*	G[		*/
/*	The number of error	*/
Sint32	errcnt;

/*	TEhhCõo[W	*/
/*	Version of sound driver			*/
Sint32	drvver;

/*	TEhCũo[W	*/
/*	Version of sound library		*/
Sint32	libver;

/***************************************************************************
 *      ֐`
 *      Function Declaration
 ***************************************************************************/

/***
 *		TEh̏
 *		Initialize sound section
 ***/

/*	TEhhCo̓]E					*/
/*	Initialize sound library and load sound driver	*/
void SoundInit(void *ptr)
{
	SDMEMBLK	memblk = NULL;

	/*	TEhCȕ				*/
	/*	Initialize Sound library				*/
	sdLibInit(NULL, 0, 0);
	/*	ubNnh				*/
	/*	Create Memory block handle  			*/
	sdMemBlkCreate(&memblk);
	/*	ubNnhݒ				*/
	/*  Setting Memory block handle  			*/
	sdMemBlkSetPrm(memblk, ptr, 0, SDD_MEMBLK_SYNC_FUNC, NULL);
	/*	TEhhCo					*/
	/*	Initialize Sound driver  				*/
	sdDrvInit(memblk);
	/*	ubN						*/
	/*	Destroy Memory block handle				*/
	sdMemBlkDestroy(memblk);
}

/***
 *			[eBeB֐
 *			Utility function
 ***/

/*	f[^̓ǂݍ	*/
/*	Load data to memory	*/
void *LoadFile(char *fname, long *len)
{
	GDFS			gdfs;
	long			fsize, nsct;
	void			*dat;

	gdfs = gdFsOpen(fname, NULL);
	if (gdfs == NULL) {
		for (;;) ;
	}
	gdFsGetFileSize(gdfs, &fsize);
	nsct = (fsize + 2047) / 2048;
	dat = syMalloc(nsct * 2048);
	if (dat == NULL) {
		for (;;) ;
	}
	gdFsRead(gdfs, nsct, dat);
	gdFsClose(gdfs);
	if (len != NULL) {
		*len = fsize;
	}

	return dat;
}

/*	Ԃւ̕ϊ															*/
/*	Convert form sampling frequency and counter to time(hour,min,sec,frame)	*/
void sfcnt2time(long sf, long ncnt, long *hh, long *mm, long *ss, long *ff)
{
	register long nf = (long)((double)ncnt/(double)sf*100.0);

	*hh = nf/(60*60*100);
	*mm = nf/(60*100) - *hh*60;
	*ss = nf/100 - *hh*60*60 - *mm*60;
	*ff = nf%100;
}

/*	TEhCuEhCõo[W擾	*/
/*	Getting sound library and sound driver			*/
void getVer(Sint32 *drvver, Sint32 *libver)
{
	Sint32	tmp;
	SDS_VER drv_ver, lib_ver;

	sdDrvGetVer(&drv_ver);
	sdLibGetVer(&lib_ver);

	tmp = (Sint32)(drv_ver.m_MajorVer);
	*drvver = tmp << 16;
	tmp = (Sint32)drv_ver.m_MinorVer;
	*drvver |= tmp << 8;
	*drvver |= (Sint32)drv_ver.m_DebugVer;

	tmp = (Sint32)(lib_ver.m_MajorVer);
	*libver = tmp << 16;
	tmp = (Sint32)lib_ver.m_MinorVer;
	*libver |= tmp << 8;
	*libver |= (Sint32)lib_ver.m_DebugVer;
	return;
}

/***
 *		TvvO
 *		Sample program
 ***/

/*	TvvȌ\		*/
/*	Show infomation of sample program	*/
void showSmpInfo(Uint32 cnt)
{
	njPrint(NJM_LOCATION( 5,  2), "[Wave Decoder]");
	njPrint(NJM_LOCATION( 5,  4), "Snd Drv   : %06x", drvver);
	njPrint(NJM_LOCATION( 5,  5), "Snd Lib   : %06x", libver);
	njPrint(NJM_LOCATION( 5,  6), "Prg Cnt   : %d", cnt);
	njPrint(NJM_LOCATION( 5,  8), "File Name : SAMPLE.WAV");
	return;
}

/*	ĐԂ̕\						*/
/*	Show status of middleware handle	*/
void showStat(MWPLY ply)
{
	Sint32	stat, tscl, cnt;
	Sint32	hh, mm, ss, ff;
	Sint8	*strstat[5] = {"STOP","PREP","EXEC","END ","ERROR"};

	hh = mm = ss = ff = 0;
	stat = mwPlyGetStat(ply);
	mwPlyGetTime(ply, &cnt, &tscl);
	sfcnt2time(tscl, cnt, &hh, &mm, &ss, &ff);
	njPrint(NJM_LOCATION( 5,  9), "Frequency : %d", tscl);
	njPrint(NJM_LOCATION( 5, 10), "Status    : %s", strstat[stat]);
	njPrint(NJM_LOCATION( 5, 11), "Time      : %02d:%02d.%02d", mm, ss, ff);
	return;
}

Sint32 pause = OFF;

/*	WAVEt@C̍Đ	*/
/*	Playing WAVE file	*/
void wavPlay(char *fname)
{
	MWPLY			ply;
	MWS_PLY_CPRM	cprm;
	Sint32			stat;
	Sint32			cnt;
	const PDS_PERIPHERAL	*per;

	/*	nh						*/
	/*	Create handle						*/
	cprm.size  = MWD_PLY_CALC_AWORK(MWD_PLY_MIN_AWORK);
	cprm.buf = syMalloc(cprm.size);
	ply = mwPlyCreateWav(&cprm);
	if (ply == NULL)
		for (;;) ;

	/*	ĐJn							*/
	/*	Start to play middleware			*/
	mwPlyStartFname(ply, fname);
	for (cnt=0;;cnt++) {
		per = pdGetPeripheral(PDD_PORT_A0);
		if (per->press & PDD_DGT_TX) {
			pause = ON;
			mwPlyPause(ply, ON);		/*	ꎞ~				*/
										/*	Pause					*/
		}
		if (per->press & PDD_DGT_TY) {
			pause = OFF;
			mwPlyPause(ply, OFF);		/*	ꎞ~			*/
										/*	Release pause status	*/
		}
		if (per->press & PDD_DGT_ST) {
			break;
		}
		/*	ĐԂ̊mF						*/
	    /*	Check status of middleware			*/
		stat = mwPlyGetStat(ply);
		if ((stat == MWE_PLY_STAT_PLAYEND) || (stat == MWE_PLY_STAT_ERROR)) {
			break;
		}

		showSmpInfo(cnt);
		showStat(ply);

		/*	~hEFÃT[o֐			*/
		/*	Server function of middleware		*/
		mwPlyExecServer();
		njWaitVSync();
	}
	/*	nh						*/
	/*	Destroy handle						*/
	mwPlyDestroy(ply);
	njWaitVSync();
	syFree(cprm.buf);
	return;
}

/***
 *		C
 *		MAIN
 ***/

/*	AvP[VI	*/
/*	Finalize application		*/
void usrExit(void)
{
	/*	CuI					*/
	mwPlyFinishWav();
	/*	ECȕI				*/
	sbExitSystem();
	/*	VvvCփWv			*/
	syBtExit();
}

/*	GDt@CVXẽG[ɋN֐			*/
/*	Callback function when an error occur in GD file system	*/
void usrGdErrFunc(void *obj, Sint32 errcode)
{
	if (errcode == GDD_ERR_TRAYOPEND || errcode == GDD_ERR_UNITATTENT) {
		/*	AvP[VI	*/
		/*	Finalize application		*/
		usrExit();
 	}
 	return;
}

/*	[UV-SYNC荞݂ɓo^֐	*/
/*	Function that user entry into V-Int		*/
void usrVsyncFunc(void)
{
	Sint32 dstat;

    /* TEhCũT[o */
    sdSysServer();
	/* GDhCȕԃ`FbN */
	dstat = gdFsGetDrvStat();
	if (dstat == GDD_DRVSTAT_OPEN || dstat == GDD_DRVSTAT_BUSY) {
		gdFsReqDrvStat();
	}
	return;
}

/*	~hEFÃG[ɋN֐			*/
/*	Callback function when an error occur in middleware	*/
void usrMwErrFunc(void *obj, Sint32 ecode)
{
    if (ecode != MWD_PLY_ERR_OK) {
		errcnt++;
    }
}

void	main(void)
{
	void	*snddrv;

	/*	ECȕ	*/
	/*	Initialize SHINOBI		*/
    njSetBorderColor(0x00000000) ;
	sbInitSystem(NJD_RESOLUTION_320x240_NTSCNI,NJD_FRAMEBUFFER_MODE_RGB565,1);
	njInitVertexBuffer(30000, 0, 30000, 0, 0);
    njInitPrint(NULL, 0, 0);
	/*	GDt@CVXeG[R[obN֐̓o^	*/
	/*	Entry callback function of GD file system error	*/
	gdFsEntryErrFuncAll((void *)usrGdErrFunc, NULL);

	/*	TEhCȕ	*/
	/*	Prepare sound library		*/
	snddrv = LoadFile("manatee.drv", NULL);
	SoundInit(snddrv);
	getVer(&drvver, &libver);
	/*	DMA ][h				*/
	/*	Set transfer mode to DMA  	*/
	sdMemBlkSetTransferMode(SDE_MEMBLK_TRANSFER_MODE_DMA);

	njSetVSyncFunction((void *)usrVsyncFunc);

	mwPlyInitWav(MWD_PLY_SVR_VSYNC);
	mwPlyEntryErrFunc((void *)usrMwErrFunc, NULL);

	for (;;) {
		wavPlay("SAMPLE.WAV");				/*	WAVEt@C̍Đ	*/
											/*	Playing WAVE file	*/
	}

	/*	AvP[VI	*/
	/*	Finalize application		*/
	usrExit();
}


/* end of file */
