/********************************************************************
 *  Shinobi Library Sample
 *  Copyright (c) 1998 SEGA
 *
 *  Library : Controller Library
 *  Module  : Gun Functions (Continuous shot sample)
 *  File    : gun.c
 *  Date    : 1999-03-03
 *  Version : 1.00
 ********************************************************************/

#include "gun.h"


/******** KW␳pp[^ *****************************************/
/******** Gun position adjust parameter.                                   */
typedef struct {
	Sint32 min;             /* Wŏl                                  */
	                        /* Minimum position                            */
	Sint32 cen;             /* Wl                                  */
	                        /* Center position                             */
	Sint32 max;             /* Wől                                  */
	                        /* Maximum position                            */
} ADJPARAM;


/******** Kp[N *****************************************************/
/******** Work area for gun.                                               */
typedef struct {
	Sint32 rawX, rawY;      /* KW(f[^)                          */
	                        /* Gun position(raw data)                      */
	Sint32 x, y;            /* KW(␳)                            */
	                        /* Gun position(corrected)                     */
	Sint32 rawOnScreen;     /* KʓĂ邩(f[^)        */
	                        /* Does the muzzle point to the inside of      */
	                        /* a screen ? (raw data)                       */
	Sint32 OnScreen;        /* KʓĂ邩                  */
	                        /* Does the muzzle point to the inside of      */
	                        /* a screen ?                                  */
	Sint32 trigger;         /* gK                                  */
	                        /* Is the trigger pressed ?                    */
	ADJPARAM adjx, adjy;    /* KW␳pp[^                    */
	                        /* Gun adjust parameters                       */
} GUNWORK;


static GUNWORK GunWork[4];  /* Kp[N(S|[gj                   */
                            /* Work area for gun (for 4 control ports)     */
static GUNINFO GunInfo[4];  /* K(S|[gj                       */
                            /* Gun information (for 4 control ports)       */
static Sint32 RapidFlag[4]; /* A˃tO(S|[g)                      */
                            /* Continuous shot flag(for 4 control ports)   */


/*===============================================================*/
/* ␳̃KWƂ肤ŏAőASl                  */
/* ̒lύXƁA␳̃KẄړ͈͂ύXł܂B*/
/* ܂ANTSC̏ꍇ͉ʎӂ\ȂƂlāA  */
/* ͈͂Ă܂B                                        */
/* The minumum, center and maximum value that gun coordinate     */
/* after corrected.                                              */
/* You can change a sphere of movement of gun coordinate after   */
/* corrected when change these value.                            */
/* Considers that the screen outskirts are not displayed in case */
/* NTSC and makes range narrow.                                  */
/*===============================================================*/

static const ADJPARAM ScreenAdjTblX[] = {
	/* ŏ  S  ő */
	/* min   cen   max  */
	{ -320,    0,   320},         /* VGA                                   */
	{ -312,    0,   312},         /* NTSC 640x480                          */
	{ -156,    0,   156},         /* NTSC 320x240                          */
};

static const ADJPARAM ScreenAdjTblY[] = {
	/* ŏ  S  ő */
	/* min   cen   max  */
	{  -240,   0,   240},         /* VGA                                   */
	{  -208,   0,   208},         /* NTSC 640x480                          */
	{  -104,   0,   104},         /* NTSC 320x240                          */
};


/*===============================================================*/
/* KWf[^̃Lu[Ve[u                  */
/* ̒lύXƁAẽLu[V()ł܂B*/
/* KWf[^̒l̎ɂꂼŏASAő      */
/* ܂B                                                    */
/* Calibration table of raw data of gun.                         */
/* You can do a fine adjustment of a gun when change these       */
/* values.                                                       */
/*===============================================================*/
static const ADJPARAM CalibTblX[] = {
	/* ŏ  S  ő */
	/* min  cen   max  */
	{  409,  695, 1013},          /* VGA                                   */
	{  295,  620,  891},          /* NTSC 640x480                          */
	{  295,  620,  891},          /* NTSC 320x240                          */
};

static const ADJPARAM CalibTblY[] = {
	/* ŏ  S  ő */
	/* min  cen   max  */
	{   82,  300,  519},          /* VGA                                   */
	{   45,  152,  254},          /* NTSC 640x480                          */
	{   45,  152,  254},          /* NTSC 320x240                          */
};


static const PortNo[] = {
	PDD_PORT_A0,
	PDD_PORT_B0,
	PDD_PORT_C0,
	PDD_PORT_D0,
};

/* ÕKWf[^LĂϐ */
/* Variables to memorize a lat raw coordinate of a gun. */
static Sint32 LastGunPosX;
static Sint32 LastGunPosY;

/* ␳pp[^ւ̃|C^B ʃ[hɂ؂ւ܂ */
/* Pointers to calibration table. Chances it according to screen mode. */
static const ADJPARAM* CalibX;
static const ADJPARAM* CalibY;
static const ADJPARAM* ScreenX;
static const ADJPARAM* ScreenY;

/*-------------------------------------------------------------------------*/
static Sint32 adjust(Sint32 rawval, const ADJPARAM *raw,
			const ADJPARAM *adj, Sint32 min, Sint32 max, Sint32 *onscflg);
static Sint32 adjust_sub(Sint32 val, const ADJPARAM *raw,
													const ADJPARAM *adj);
static Sint32 callback(void);

/*-------------------------------------------------------------------------*/

/*===============================================================*/
/*      Rg[CuK[hɐݒ肷             */
/*      Set up controller library in gun mode.                   */
/*===============================================================*/

void GunInit(Sint32 tvmode)
{
	pdGunEnter(PDD_GUNMODE_ALL);
	pdGunSetCallback(callback);
	pdGunSetFlashColor(0x00ffffff);

	memset(GunWork, 0, sizeof(GunWork));
	memset(GunInfo, 0, sizeof(GunInfo));
	memset(RapidFlag, 0, sizeof(RapidFlag));

	LastGunPosX = 0;
	LastGunPosY = 0;

	CalibX = &CalibTblX[tvmode];
	CalibY = &CalibTblY[tvmode];
	ScreenX = &ScreenAdjTblX[tvmode];
	ScreenY = &ScreenAdjTblY[tvmode];
}



/*
** |[gԍ\̔z̃CfbNXɕϊ
** Converts port number into an index.
*/

static Sint32 port2index(Sint32 port)
{
	switch (port) {
		case PDD_PORT_A0: return 0;
		case PDD_PORT_B0: return 1;
		case PDD_PORT_C0: return 2;
		case PDD_PORT_D0: return 3;
		default: return -1;
	}
}

/*===============================================================*/
/*      t[Ă΂K̃C֐                       */
/*      Main function of a gun called every frame.               */
/*===============================================================*/

void GunMain(void)
{
	GUNWORK* gw;
	GUNINFO* gi;
	PDS_PERIPHERAL* per;
	Sint32 gunport;
	Sint32 gunx, guny;
	Sint32 i;
	Sint32 flag1, flag2;

	for (i = 0; i < 4; i++) {
		GunWork[i].trigger = FALSE;
	}

	/* KW擾|[g𒲂ׂ           */
	/* Examine the port which got gun coordinate. */
	gunport = pdGunGetLatchedPort();

	if (gunport >= 0) {
		gw = &GunWork[port2index(gunport)];

		/* KWf[^̎擾     */
		/* Get raw coordinate of gun. */
		pdGunGetPosition(&gunx, &guny);

		if (gunx < 200) gunx += 857;

		gw->trigger= TRUE;

		/* O̒lƑSȂ΁AWȂ  */
		/* ƂāAʊOƂB                          */
		/* If a coordinate is same as a last value at all, */
		/* could not get coordinate.                       */
		if ((LastGunPosX == gunx) && (LastGunPosY == guny)) {
			gw->rawOnScreen = FALSE;
			gw->OnScreen = FALSE;
			gw->rawX = -1;
			gw->rawY = -1;
			gw->x = 999;
			gw->y = 999;
		} else {
			gw->rawOnScreen = FALSE;
			gw->rawX = gunx;
			gw->rawY = guny;
			gw->x = adjust(gw->rawX, CalibX, ScreenX, -320, 319, &flag1);
			gw->y = adjust(gw->rawY, CalibY, ScreenY, -240, 239, &flag2);
			gw->OnScreen = (flag1 && flag2) ? TRUE : FALSE;
		}

		/* ̒lۑ               */
		/* Store a coordinate of this time. */
		LastGunPosX = gunx;
		LastGunPosY = guny;
	}

	gw = GunWork;
	gi = GunInfo;
	for (i = 0; i < 4; i++, gw++, gi++) {
		/* KڑĂ邩ǂ̔ */
		/* Is gun connected ?                 */
		per = pdGetPeripheral(PortNo[i]);
		gi->IsGun = (per->info->type & PDD_DEVTYPE_LIGHTGUN) ? 1 : 0;
		/* K[NK\̂ɃRs[                   */
		/* Copy values to gun information structure from work of gun. */
		gi->x = gw->x;
		gi->y = gw->y;
		gi->rawX = gw->rawX;
		gi->rawY = gw->rawY;
		gi->trigger = gw->trigger;
		gi->OnScreen = gw->OnScreen;
	}
}


/*===============================================================*/
/*      K̏擾                                     */
/*      Get gun information.                                     */
/*===============================================================*/
const GUNINFO* GunGetInfo(Uint32 port)
{
	return &GunInfo[port];
}

/*===============================================================*/
/*      A˃[hݒ肷                                     */
/*      Set continuous shot mode.                                */
/*===============================================================*/
void GunSetRapid(Sint32 gunno, Sint32 flag)
{
	RapidFlag[gunno] = flag;
}


/*===============================================================*/
/*      Rg[Cuʏ탂[hɐݒ肷             */
/*      Set up controller library in normal mode.                */
/*===============================================================*/
void GunExit(void)
{
	pdGunLeave();
}


/*-------------------------------------------------------------------------*/


/*
** ␳ & MIN-MAX`FbN
** Linear correction and minimum-maximum check.
*/

static Sint32 adjust(Sint32 rawval, const ADJPARAM *raw, const ADJPARAM *adj,
								Sint32 min, Sint32 max, Sint32 *onscflg)
{
	Sint32 val;

	val = adjust_sub(rawval, raw, adj);

	/* ͈͊OH                   */
	/* Is a value outside of range? */
	if (val < min) {
		*onscflg = FALSE;
		return 0;
	} else if (val > max) {
		*onscflg = FALSE;
		return 0;
	}

	*onscflg = TRUE;
	return val;
}


/*
** ␳(߂l͕␳̒l)
** Linear correction (Return value is a corrected value)
*/

static Sint32 adjust_sub(Sint32 val, const ADJPARAM *raw, const ADJPARAM *adj)
{
	if (((raw->max > raw->min) ? 0 : 1)^((raw->cen > val) ? 0 : 1)) {
		if (raw->max == raw->cen) {
			return 0;
		} else {
			return (val - raw->cen) * (adj->max - adj->cen) /
									(raw->max - raw->cen) + adj->cen;
		}
	} else {
		if (raw->min == raw->cen) {
			return  0;
		} else {
			return (val - raw->cen) * (adj->min - adj->cen) /
									(raw->min - raw->cen) + adj->cen;
		}
	}
}




/*-------------------------------------------------------------------------*/

/*
** KR[obN֐
** Gun callback function.
*/

static Sint32 callback(void)
{
	static Uint32 count = 0;
	Sint32 i;
	const PDS_PERIPHERAL* per;

	for (i = 0; i < 4; i++) {
		per = pdGetPeripheral(PortNo[i]);

		if (RapidFlag[i]) {
			/* A˃tOONȂAIɃgKZbg     */
			/* If continuous shot flag is on, sets trigger regularly. */
			if (per->on & PDD_DGT_TA) {
				if (((count >> 1) & 3) == i) {
					pdGunSetTrigger(PortNo[i]);
				}
			}
		} else {
			/* A˃tOOFFȂAۂ̃gKŃgKZbg */
			/* If continuous shot flag is FALSE, sets trigger with       */
			/* real trigger.                                             */
			if (per->press & PDD_DGT_TA) {
				pdGunSetTrigger(PortNo[i]);
			}
		}
	}

	count++;

	return FALSE;
}

