/********************************************************************
 *  Shinobi Library Sample
 *  Copyright (c) 1998 SEGA
 *
 *  Library : Controller Library
 *  Module  : Controller distinction
 *  File    : perchk.c
 *  Date    : 1999-01-28
 *  Version : 1.01
 *  Note    :
 ********************************************************************/

#include "perchk.h"


/*===============================================================*/
/* Rg[̎Ă{^                                */
/* Which button does controller have ?                           */
/*===============================================================*/

#define BT(a)     PDD_DEV_SUPPORT_##a


/* WRg[ɕtĂ{^ */
/* Dreamcast controller               */
#define BT_CONTROLLER   (BT(TA)  | BT(TB)  | BT(TX)  | BT(TY)  | BT(ST)  | \
                         BT(KU)  | BT(KD)  | BT(KL)  | BT(KR)  | \
                         BT(AX1) | BT(AY1) | BT(AL)  | BT(AR))

/* [VORg[ɕtĂ{^ */
/* Racing controller                      */
#define BT_RACING       (BT(TA)  | BT(TB)  | BT(ST)  | \
                         BT(KU)  | BT(KD)  | BT(KL)  | BT(KR)  | \
                         BT(AX1) | BT(AL)  | BT(AR))

/* A[P[hXeBbNɕtĂ{^ */
/* Arcade stick                           */
#define BT_STICK        (BT(TA)  | BT(TB)  | BT(TC)  | \
                         BT(TX)  | BT(TY)  | BT(TZ)  | BT(ST)  | \
                         BT(KU)  | BT(KD)  | BT(KL)  | BT(KR))

/* ނRg[ɕtĂ{^ */
/* Fishing controller                 */
#define BT_FISHING      (BT(TA)  | BT(TB)  | BT(TX)  | BT(TY)  | BT(ST)  | \
                         BT(KU)  | BT(KD)  | BT(KL)  | BT(KR)  | \
                         BT(AX1) | BT(AY1) | BT(AL)  | BT(AR)  | \
                         BT(AX2) | BT(AY2) | BT(AL)  | BT(AR))



/*===============================================================*/
/* ytFʃe[u                                      */
/*   Ẽe[uɂ鏇ԂǂɁAĂ{^𔻒肵  */
/*     ǂ̃Rg[ڑĂ邩𔻕ʂ܂B          */
/*   E{^̑Rg[ɑΉ邽߁A{^  */
/*      "==" ōs킸Arbg}XNčs܂B      */
/*   Ẽe[uł́AA[P[hXeBbNŗDŔ肵A  */
/*     ނRAWRg[AnhƑ܂B          */
/*   EAvP[VɂāÃe[uύXKv    */
/*     ł傤B                                            */
/*   E{IɁA{^̑ǐjRg[  */
/*     肷悤ȃe[uɂĂB                    */
/*   EނŔAWRg[ɂ{^ׂĎ  */
/*     ނRΉ\tgȂΕWRg[Dx    */
/*     AΉ\tgȂΒႭĂB                */
/*   EL[{[hAKRg[͂̃e[uƂ͕ʂɔ  */
/*     s܂B                                                */
/*   EAvP[V̐pRg[𔻕ʂꍇ  */
/*     ́A܂̃e[u𗘗pʂsOɃytF  */
/*     ̓ŐpRg[ǂ𔻕ʂĂBp  */
/*     Rg[łȂꍇ́Ãe[uŔʂs      */
/*     B                                                */
/*===============================================================*/

typedef struct {
	Uint32 no;                           /* Peripheral No,       */
	Uint32 button;                       /* Buttons              */
} CONTROLLER;


static const CONTROLLER ControllerTbl[] = {
	{PER_STICK,      BT_STICK},          /* Arcade Stick         */
	{PER_FISHING,    BT_FISHING},        /* Fishing Controller   */
	{PER_CONTROLLER, BT_CONTROLLER},     /* Dreamcast Controller */
	{PER_RACING,     BT_RACING},         /* Racing Controller    */
};

#define TABLE_SIZE (sizeof(ControllerTbl) / sizeof(ControllerTbl[0]))

/*===============================================================*/
/* ytF̔                                            */
/*   E܂ڑǂ𔻒肵܂BɃL[{[hǂA  */
/*     KRg[ǂ𔻒肵܂B                    */
/*   EKRg[́ACgKƃRg[́AQ    */
/*     ^Cv𕹂܂Ał̓CgK^Cv    */
/*     Ƃɂ蔻ʂsĂ܂B                            */
/*   ERg[^Cvłꍇɂ́AĂ{^    */
/*     `FbNARg[̎ʂ𔻕ʂ܂B              */
/* Peripheral distinction                                        */
/*===============================================================*/

Sint32 CheckController(Uint32 port)
{
	const PDS_PERIPHERAL* per;
	const CONTROLLER* ctl;
	Sint32 i;

	per = pdGetPeripheral(port);

	ctl = ControllerTbl;

	if (per->info->type == 0) {
		/* ڑĂȂ     */
		/* No controller connected. */
		return PER_NONE;
	} else if (per->info->type & PDD_DEVTYPE_KEYBOARD) {
		/* L[{[h       */
		/* Keyboard is connected. */
		return PER_KEYBOARD;
	} else if (per->info->type & PDD_DEVTYPE_LIGHTGUN) {
		/* KRg[       */
		/* Gun controller is connected. */
		return PER_GUN;
	} else if (per->info->type & PDD_DEVTYPE_CONTROLLER) {
			/* Rg[n̂ŁA{^𔻕 */
			/* Controller is connected. Check button. */
		for (i = 0; i < TABLE_SIZE; i++, ctl++) {
			if ((per->support & ctl->button) == ctl->button) {
				return ctl->no;
			}
		}
	}

	/* e[uɂȂRg[ڑĂ       */
	/* Controller which is not on a table is connected. */
	return PER_UNKNOWN;
}


