/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F ݒ肷
              sets the ambient light
              
  ҏW F 98-04-28 Jn
              98-06-19 SET5Ή
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 10
#define MAX_VERTEXES 4096

#define COLOR_CHANGE_CYCLE 500 					/* F\ */
												/* a cycle of display one color*/
#define CUBE_MAX 3 								/* ̂̐ */
												/* the number of cube */
#define CUBE_ROTATION_SPEED NJM_DEG_ANG(1.0F) 	/* ]x */
												/* the velosity of rotation */
extern NJS_OBJECT object_cube_root[];

static NJS_MATRIX *TranslationMatrix;
static NJS_VERTEX_BUF *VertexBuffer;

static NJS_VIEW View;
static NJS_LIGHT AmbientLight;

static PDS_PERIPHERAL *key;
/*----------------------------------------------------------------------
           
----------------------------------------------------------------------*/
void njUserInit(void)
{
    /* VXȅ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* msrbʂ̏c␳ */
	/* corrects the screen aspect ratio */
    njSetAspect(1.0F, 0.45F);

    /* 蓖 */
    /* allocates the memory */
    TranslationMatrix = syMalloc(sizeof(NJS_MATRIX)     * MAX_MATRIX_DEPTH);
    VertexBuffer      = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* wiF̐ݒ */
	/* sets background color */
    njSetBackColor(0x00C00000, 0x0000C000, 0x000000C0);

    /* _i[obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
    
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bN̏ */
	/* initializes the matrix stack */
    njInitMatrix(TranslationMatrix, MAX_MATRIX_DEPTH, 1);

    /* 3DVXȅ */
    /* initializes 3D system  */ 
    njInit3D(VertexBuffer, MAX_VERTEXES);

    /* ̏ */
    /* initializes the light source */
    njCreateLight(&AmbientLight, NJD_AMBIENT);
    njSetLightIntensity(&AmbientLight, 1.0F, 1.0F, 1.0F);
    njSetLightAlpha(&AmbientLight, 1.0F);

    /* r[̏ */
    /* initializes the user-defined view */
    njInitView(&View);

    /* r[̐ݒ */
    /* sets the user-defined view as the current view */
    njSetView(&View);

	/* ̐Fw */
	/* sets the color of debug printing */
	njPrintColor(0xFFC0C0C0);
}

/*----------------------------------------------------------------------
         C
----------------------------------------------------------------------*/
Sint32 njUserMain(void)
{
    /* ϐ̏ݒ */
    /* initializes the valiables */
    static Angle rotateAng = NJM_DEG_ANG(0.0F);

    int cubeNo = 0;        	/* ̂Ԃɕ\邽߂Ɏgϐ */
    static int counter = 0;	/* F\鎞Ԃ𒲐邽߂̃JE^ϐ */
	static Bool pause = FALSE; 	/* |[YĂ邩 */
								/* pause or not           */
	key = pdGetPeripheral(PDD_PORT_A0);

	/* |[Y̔    */
	/* checks the pause */
	if (key->press & PDD_DGT_ST) {
	    if (pause == FALSE) {
		    njSetBackColor(0x00000000, 0x00000000, 0x00000000);
		    pause = TRUE;
		} else {
		    njSetBackColor(0x00C00000, 0x0000C000, 0x000000C0);
		    pause = FALSE;
		}
	}

    /* F̕\ */
    /* prints the name of color */
	if (pause == TRUE) {
	    njPrintC(NJM_LOCATION( 4, 4),"PAUSE");
		njSetLightColor(&AmbientLight, 0.5F, 0.5F, 0.5F);
	} else {
	    counter++;
		if (counter == COLOR_CHANGE_CYCLE * CUBE_MAX) {
		  counter = 0;
		}
		if (counter < COLOR_CHANGE_CYCLE) {
		    njPrintC(NJM_LOCATION( 4, 4),"COLOR RED");
			njSetLightColor(&AmbientLight, 1.0F, 0.0F, 0.0F);
		} else if (counter < COLOR_CHANGE_CYCLE * 2) {
		    njPrintC(NJM_LOCATION( 4, 4),"COLOR GREEN");
			njSetLightColor(&AmbientLight, 0.0F, 1.0F, 0.0F);
		} else {
		    njPrintC(NJM_LOCATION( 4, 4),"COLOR BLUE");
			njSetLightColor(&AmbientLight, 0.0F, 0.0F, 1.0F);
		}
	}
    /* Tv̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 2), "Ninja Sample LIGHT COLOR CHANGE");

    /* ̂̕\  */
    /* draw the cube */
 
    for (cubeNo = 0; cubeNo < CUBE_MAX; cubeNo++){
        njPushMatrix(NULL);
        njTranslate(NULL, (cubeNo - 1) * 3.0F, 0.0F, -10.0F);
        /* X,Y,Z ̏Ԃɉ] */
        /* rotates the object   */
        njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);

        /* IuWFNg̕` */
        /* draws the object   */
        njDrawObject(object_cube_root);
        njPopMatrix(1);
    }
    /* X,Y,Z ւ̉]p̍XV */
    /* update the x,y,z angel   */
    rotateAng += CUBE_ROTATION_SPEED;

    return NJD_USER_CONTINUE;
}

/*----------------------------------------------------------------------
         I
----------------------------------------------------------------------*/
void njUserExit(void)
{
	njExitPrint();

    /* ̊J */
	/* frees the memory area */
    syFree(TranslationMatrix);
    syFree(VertexBuffer);

    /* VXȅI */
    /* exits the system    */
    sbExitSystem();
}
