/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F oCjAtB^ݒ肷
              uses the bilinear filter
  ҏW F 98-04-28 Jn
              98-06-19 SET5Ή
              98-09-16 eNX`obt@mۂ̃TCYύX
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}


/* O[of[^ZNV */
/* Global Data                */
#define VERTEX_MAX      1028
#define MATRIX_DEPTH    12

/* 3D֘Af[^ */
/* 3D Data      */
static NJS_MATRIX      *g_Matrix;
static NJS_VERTEX_BUF  *g_VBuf;

static NJS_VIEW    g_View;
static NJS_LIGHT   g_Light;

/* _f[^  */
/* Vertex Data */
static NJS_POINT3COL  g_PointCol;
static NJS_POINT3     g_Point[4];
static NJS_COLOR      g_Color[4];
static NJS_COLOR      g_UV[4];

/* eNX`f[^ */
/* Texture Data     */
#define TEXTURE_NUM 1
#define TEXTURE_BUFFER_SIZE 0x800*10

Sint8   g_TexBuf[TEXTURE_BUFFER_SIZE];
static NJS_TEXNAME    texname = {"tex.pvr", NJD_TEXATTR_GLOBALINDEX, 0};
static NJS_TEXLIST    texlist = {&texname, TEXTURE_NUM};
static NJS_TEXMEMLIST tex[TEXTURE_NUM];

/* |[Y֐ */
/* function for PAUSE */
void PauseProc(void);

/* |[YptO */
/* flag for PAUSE */
Bool g_bPause = FALSE;

/**************************************************************************

    njUserInit --- NinjaVXe֐

**************************************************************************/

void njUserInit(void)
{
    /* VXe */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem( NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* msrbʂ̏c␳ */
	/* corrects the screen aspect ratio */
    njSetAspect(1.0F, 0.45F);


    /* ̊蓖 */
    /* allocates the memory */
    g_Matrix = (NJS_MATRIX *)syMalloc(sizeof(NJS_MATRIX) * MATRIX_DEPTH);
    g_VBuf = (NJS_VERTEX_BUF *)syMalloc(sizeof(NJS_VERTEX_BUF) * VERTEX_MAX);

    /* eNX`̃obt@m */
    /* allocates the buffers for registration of vertex data */
    njInitTextureBuffer((Sint8 *)g_TexBuf,TEXTURE_BUFFER_SIZE);

    /* _obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(VERTEX_MAX * 64 / 4, 0, VERTEX_MAX * 64 / 4, 0 ,0);

	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* 3DVXe */
    /* initializes 3D system  */ 
    njInitMatrix(g_Matrix, MATRIX_DEPTH, 1);
    njInit3D(g_VBuf, VERTEX_MAX);

    /* r[̏ */
    /* initializes the user-defined view */
    njInitView(&g_View);
    njSetView(&g_View);

    /* Cg̏ */
    /* creates the light source */
    njCreateLight(&g_Light, NJD_LAMBERT_DIR);

    /* eNX`̏ */
    /* initializes the texture */
    njInitTexture(tex, TEXTURE_NUM);
    njLoadTexture(&texlist);
    njSetTexture(&texlist);
    njSetTextureNum(0);

    /* _f[^̏ */
    /* sets the vertex data */
    g_PointCol.p   = g_Point;
    g_PointCol.col = g_Color;
    g_PointCol.tex = g_UV;
    g_PointCol.num = 4;
    
    g_Point[0].x = -1.0F;
    g_Point[0].y =  1.0F;
    g_Point[0].z =  0.0F;
    g_UV[0].tex.u = 0;
    g_UV[0].tex.v = 0;
    g_Color[0].argb.a = 0xff;
    g_Color[0].argb.r = 0xff;
    g_Color[0].argb.g = 0xff;
    g_Color[0].argb.b = 0xff;

    g_Point[1].x = -1.0F;
    g_Point[1].y = -1.0F;
    g_Point[1].z =  0.0F;
    g_UV[1].tex.u = 0;
    g_UV[1].tex.v = 0xff;
    g_Color[1].argb.a = 0xff;
    g_Color[1].argb.r = 0xff;
    g_Color[1].argb.g = 0xff;
    g_Color[1].argb.b = 0xff;

    g_Point[2].x =  1.0F;
    g_Point[2].y = -1.0F;
    g_Point[2].z =  0.0F;
    g_UV[2].tex.u = 0xff;
    g_UV[2].tex.v = 0xff;
    g_Color[2].argb.a = 0xff;
    g_Color[2].argb.r = 0xff;
    g_Color[2].argb.g = 0xff;
    g_Color[2].argb.b = 0xff;

    g_Point[3].x =  1.0F;
    g_Point[3].y =  1.0F;
    g_Point[3].z =  0.0F;
    g_UV[3].tex.u = 0xff;
    g_UV[3].tex.v = 0;
    g_Color[3].argb.a = 0xff;
    g_Color[3].argb.r = 0xff;
    g_Color[3].argb.g = 0xff;
    g_Color[3].argb.b = 0xff;

    /* obNOEh̃J[ݒ */
   	/* sets background color */
    njSetBackColor(ARGB(0xff, 0xc0, 0, 0 ), 
                   ARGB(0xff, 0, 0xc0, 0 ), 
                   ARGB(0xff, 0, 0, 0xc0 ));

    /* fobO̕Fݒ */
  	/* sets the color of debug printing */
    njPrintColor(ARGB(0xff, 0xc0, 0xc0, 0xc0));
}


/**************************************************************************

    njUserMain --- NinjaVXeC֐

**************************************************************************/

Sint32 njUserMain(void)
{
    NJS_PERIPHERAL* per;
    static Angle ang = NJM_DEG_ANG(0.0F);
    static int time = 0;

    /* WCXeBbN|[gytF擾 */
    /* gets the peripheral information              */
    per = njGetPeripheral(NJD_PORT_A0);    

    /* {^ꂽǂ`FbN */
    /* checks the START button          */
    if (per->press & NJD_DGT_ST) {
        g_bPause = !g_bPause;
    }

    /* |[YȂȂ */
    /* not pause ?        */
    if (!g_bPause) {
        /* TbƂɃeNX`tB^[h؂ւ */
        /* changes the texture filter every 5 minitus     */
        if (++time > 600) {
            time = 0;
        }
        if (time < 300) {
            njTextureFilterMode(NJD_TEXTUREFILTER_BILINEAR);
            njPrintC(NJM_LOCATION(30, 23), "Bilinear Filter = ON");
        } else {
            njTextureFilterMode(NJD_TEXTUREFILTER_POINT_SAMPLE);
            njPrintC(NJM_LOCATION(30, 23), "Bilinear Filter = OFF");
        }

        /* }gNX̃NA */
        /* clears the matrix stack */
        njClearMatrix();

        /* ړ  */
        /* translate */
        njTranslate(NULL, 0.0F, 0.0F, -5.0F); 

        /* ] */
        /* roteate  */
        ang += NJM_DEG_ANG(1.0F); 
        njRotateXYZ(NULL, NJM_DEG_ANG(0.0), ang, NJM_DEG_ANG(0.0));
    /* |[YȂ */
    /* pause ?    */
    } else {
        PauseProc();
        njPrintC(NJM_LOCATION(38, 25), "PAUSE");
    }

    /* ` */
    /* Draw */
    njDrawPolygon3D(&g_PointCol, 4, NJD_FILL | NJD_USE_TEXTURE);

    /* fobO\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 4), "Ninja Sample BI-LINEAR FILTER");

    return NJD_USER_CONTINUE;
}


/**************************************************************************

    PauseProc --- |[Y֐

**************************************************************************/

void PauseProc(void)
{
    NJS_POINT2COL pc;
    NJS_POINT2    p[4];
    NJS_COLOR     col[4];
        
    /* pXvCg̐ݒ */
    /* sets the translucent splits */
    pc.p   = p;
    pc.col = col;
    pc.tex = NULL;
    pc.num = 4;
    
    p[0].x = 0;
    p[0].y = 0;
    col[0].color = 0xc0000000;

    p[1].x = 0;
    p[1].y = 480;
    col[1].color = 0xc0000000;

    p[2].x = 640;
    p[2].y = 480;
    col[2].color = 0xc0000000;

    p[3].x = 640;
    p[3].y = 0;
    col[3].color = 0xc0000000;

    /* |S̕` */
    /* draws the translucent polygons */
    njDrawPolygon2D(&pc, 4, -2.0F, NJD_TRANSPARENT | NJD_FILL);
}


/**************************************************************************

    njUserExit --- NinjaVXeI֐

**************************************************************************/

void njUserExit(void)
{
	njExitPrint();
    njExitTexture();

    /* ̊J */
   	/* frees the memory area */
    syFree(g_Matrix);
    syFree(g_VBuf);
    
    /* VXeI֐ */
    /* exits the system    */
    sbExitSystem();
}
