/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F |S̎c
              afterimage effects of polygon
  ҏW F 98-05-07 Jn
              98-06-19 SET5Ή
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 20
#define MAX_VERTEXES  30000

#define BALL_MAX 1             	              /* {[̐             */
                                              /* the number of balls    */
#define FRAMES_PER_SECOND 60                  /* PbԂ̃t[     */
                                              /* the number of frame per second */
#define FRAME_TIME (1.0F / FRAMES_PER_SECOND) /* Pt[̎       */
                                              /* the time of 1 frame    */

#define BALL_RADIUS    15.0F                  /* ̔a               */
                                              /* the radius of the ball */
#define START_SPEED   500.0F                  /* x               */
                                              /* the initial speed      */
#define GRAVITY       980.0F                  /* d͉xG            */
                                              /* gravity acceleration   */
#define EFFECTSIZE     21                     /* ۑcf[^̐ */
                                              /* the number of the afterimage */
#define FLOOR_HEIGHT -BALL_RADIUS             /* nʂ̍             */
                                              /* the height of floor    */
#define FLOOR_WIDTH   80.0F                   /* nʂ̕               */
                                              /* the width of the floor */
#define FLOOR_LONG  1000.0F                   /* nʂ̉s           */
                                              /* the length of the floor*/
#define FLOOR_COLOR 0x77C0C0C0                /* nʂ̐F               */
											  /* the color of the floor */
void PauseChange(void);
void DrawFloor  (void);
void DrawBall   (void);
void DrawEffect (void);
void DrawShadow (void);

extern NJS_CNK_OBJECT object__sphere3_sphere3[];

typedef struct BallData;
struct BallData {
    Float x;  /* XW     */			  /* x coordinate of the ball */
    Float y;  /* YW     */			  /* y coordinate of the ball */
    Float z;  /* ZW     */             /* z coordinate of the ball */
    Float vx; /* Xx */             /* x speed of the ball      */
    Float vz; /* Yx */             /* y speed of the ball      */
};

static NJS_MATRIX     *Matrix;
static NJS_VERTEX_BUF *VertexBuffer;

static NJS_VIEW  View;
static NJS_LIGHT Light;

struct BallData   BallPos[BALL_MAX];               /* {[̃f[^ */
                                                   /* the ball data  */
struct BallData	  BallBackup[BALL_MAX];			   /* ۑpf[^          */
                                                   /* the save data of ball */
static NJS_POINT3 BallEffect[BALL_MAX][EFFECTSIZE];/* Wf[^          */
                                                   /* the coordinate data */

Float BallTime = 0.0F; 	/*  */
                        /* time */
Float BackupTime;		/* ۑp */
PDS_PERIPHERAL *key;	/* {^ */
                        /* peripheral information */

static Bool pause = FALSE;		/* |[YĂ邩ǂ */
                                /* pause(TRUE) or not(FALSE)    */
/*----------------------------------------------------------------------
           
           initialization
----------------------------------------------------------------------*/
void njUserInit(void)
{
    /* VXȅ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* msrbʂ̏c␳ */
	/* corrects the screen aspect ratio */
    njSetAspect(1.0F, 0.45F);

    /* ̊蓖 */
    /* allocates the memory */
    Matrix       = syMalloc(sizeof(NJS_MATRIX) * MAX_MATRIX_DEPTH);
    VertexBuffer = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* wiF̐ݒ */
  	/* sets background color */
    njSetBackColor(ARGB(0, 192, 192, 0), ARGB(0, 192, 0, 192), ARGB(0, 0, 192, 192));

    /* _i[obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bN̏ */
   	/* initializes the matrix stack */
    njInitMatrix(Matrix, MAX_MATRIX_DEPTH, 1);

    /* 3DVXȅ */
    /* initializes 3D system  */ 
    njInit3D(VertexBuffer, MAX_VERTEXES);

    /* }eAύXł悤ɐݒ */
    /* the material is changeable */
    njControl3D(NJD_CONTROL_3D_CONSTANT_MATERIAL | NJD_CONTROL_3D_CNK_CONSTANT_ATTR);
    njSetConstantAttr(0xFFFFFFFF, NJD_FST_UA);

    /* ̏ */
    /* initializes the light source */
    njCreateLight(&Light, NJD_SPEC_DIR);
    njSetLightIntensity(&Light, 1.0F, 1.0F, 1.0F);
    njSetLightColor(&Light, 1.0F, 1.0F, 1.0F);
    njSetLightDirection(&Light, -1.0F, -1.0F, -1.0F);
    njSetLightAlpha(&Light, 0.5F);

    /* r[̏ */
    /* initializes the user-defined view */
    njInitView(&View);

    /* r[̐ݒ */
    /* sets the user-defined view as the current view */
    njSetView(&View);

    /* ̍W̏ */
    /* initializes the coordinate of the ball */
    BallPos[0].x  =    0.0F;
    BallPos[0].y  =    0.0F;
    BallPos[0].z  = -100.0F;
    BallPos[0].vx =    1.0F;
    BallPos[0].vz =    1.0F;

	/* F̐ݒ */
	/* sets the color of debug printing */
	njPrintColor(0xFFC0C0C0);
}

/*----------------------------------------------------------------------
         C
         main
----------------------------------------------------------------------*/
Sint32 njUserMain(void)
{
    /* Tv̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 4), "Ninja Sample BOUND");
	if (pause == TRUE) {
	    njPrintC(NJM_LOCATION(30, 5), "PAUSE");
	}

    /* }gbNXX^bÑNA */
    /* clears the matrix stack      */
    njClearMatrix();
    
	/* |[Ỹ`FbN */
	/* checks the pause */
	key = pdGetPeripheral(PDD_PORT_A0);
	if (key->press & PDD_DGT_ST) {
	    PauseChange();
	}
	
    /* ̕` */
    /* draws the floor */
    DrawFloor();
    
    /* ̏ */
    /* draws the balls */
    DrawBall();

    /* c̏ */
    /* draws the afterimage */
	if (pause == FALSE) {
	    DrawEffect();
	}
	
    /* ȅ */
    /* draws the shadow */
    DrawShadow();

    /* Ԃ̌o */
    /* update the time value */
    BallTime += FRAME_TIME;

    return NJD_USER_CONTINUE;
}

/*----------------------------------------------------------------------
            |[Y̐؂ւ
            pause 
----------------------------------------------------------------------*/
void PauseChange(void)
{
    if (pause == FALSE) {
	    pause = TRUE;
		BallBackup[0].x = BallPos[0].x;
		BallBackup[0].y = BallPos[0].y;
		BallBackup[0].z = BallPos[0].z;
		BallBackup[0].vx = BallPos[0].vx;
		BallBackup[0].vz = BallPos[0].vz;
		BackupTime = BallTime;
		BallPos[0].vx = 0.0F;
		BallPos[0].vz = 0.0F;
	} else {
	    pause = FALSE;
		BallPos[0].x = BallBackup[0].x;
		BallPos[0].y = BallBackup[0].y;
		BallPos[0].z = BallBackup[0].z;
		BallPos[0].vx = BallBackup[0].vx;
		BallPos[0].vz = BallBackup[0].vz;
		BallTime = BackupTime;
	}
}

/*----------------------------------------------------------------------
            ̕`
            draws the floor
----------------------------------------------------------------------*/
void DrawFloor(void)
{
    NJS_POINT3COL point;
    NJS_POINT3 pointData[4];
    NJS_COLOR  colorData[4];

    /* _WAF̐ݒ */
    /* vertex coordinates, color */
    pointData[0].x = -FLOOR_WIDTH;
    pointData[0].y =  FLOOR_HEIGHT;
    pointData[0].z = -FLOOR_WIDTH;
    colorData[0].color = FLOOR_COLOR;

    pointData[1].x = -FLOOR_WIDTH;
    pointData[1].y =  FLOOR_HEIGHT;
    pointData[1].z = -FLOOR_LONG;
    colorData[1].color = FLOOR_COLOR;

    pointData[2].x =  FLOOR_WIDTH;
    pointData[2].y =  FLOOR_HEIGHT;
    pointData[2].z = -FLOOR_LONG;
    colorData[2].color = FLOOR_COLOR;

    pointData[3].x =  FLOOR_WIDTH;
    pointData[3].y =  FLOOR_HEIGHT;
    pointData[3].z = -FLOOR_WIDTH;
    colorData[3].color = FLOOR_COLOR;

    point.p = pointData;
    point.col = colorData;

    /* ̕\ */
	/* draws the floor */
    njPushMatrix(NULL);
        njDrawPolygon3D(&point, 4, NJD_FILL | NJD_TRANSPARENT);
    njPopMatrix(1);
}

/*----------------------------------------------------------------------
            ̏
            draw the ball
----------------------------------------------------------------------*/
void DrawBall(void)
{
    NJS_ARGB argb;

    /* XWiEj̐ݒ */
    /* sets the x coordinate */
    BallPos[0].x += BallPos[0].vx;
    if (BallPos[0].x > FLOOR_WIDTH || BallPos[0].x < -FLOOR_WIDTH ) {
        BallPos[0].vx = -BallPos[0].vx;
    }/* ͂˂*/
     /* bounds    */ 
 
    /* YWij̐ݒ */
    /* sets the y coordinate */
    /* h= v0t - 1/2 gt^2 */
    BallPos[0].y = (START_SPEED * BallTime) - (0.5F * GRAVITY * BallTime * BallTime);
    if (BallPos[0].y < (FLOOR_HEIGHT + BALL_RADIUS)) {  
        BallTime = 0.0F;
    }/* ͂˂ */
     /* bounds    */ 

    /* ZWisj̐ݒ */
    /* sets the z coordinate */
    BallPos[0].z += BallPos[0].vz;
    if (BallPos[0].z > -FLOOR_WIDTH || BallPos[0].z < -FLOOR_LONG ) {
        BallPos[0].vz = -BallPos[0].vz;
    }/* ͂˂ */
     /* bounds    */ 

    /* }eA̐ݒi̐F͔j*/
    /* sets the material color */
    argb.a = 1.0F;
    argb.r = 1.0F;
    argb.g = 1.0F;
    argb.b = 1.0F;
    njSetConstantMaterial(&argb);

    /* ʂ̕\ */
    /* draws the ball */
    njPushMatrix(NULL);
        njTranslate(NULL, BallPos[0].x, BallPos[0].y, BallPos[0].z);
        njCnkDrawObject( object__sphere3_sphere3 );
    njPopMatrix(1);
}

/*----------------------------------------------------------------
            ȅ
            draws the shadow
----------------------------------------------------------------*/
void DrawShadow(void)
{
    NJS_ARGB argb;
    /* }eA̐ݒie̐F͍j */
    /* set the material color */
    argb.a = 1.0F;
    argb.r = 0.0F;
    argb.g = 0.0F;
    argb.b = 0.0F;
    njSetConstantMaterial(&argb);
    
    /* e̕\ */
    /* draws the shadow */
    njPushMatrix(NULL);
        njTranslate(NULL, BallPos[0].x, FLOOR_HEIGHT, BallPos[0].z);
        njScale(NULL, 1.0F, 0.001F, 1.0F);
        njCnkDrawObject( object__sphere3_sphere3 );
    njPopMatrix(1);
}

/*----------------------------------------------------------------- 
            c̏
            draws the afterimage
------------------------------------------------------------------*/
void DrawEffect(void)
{
    NJS_ARGB argb;
    int effectnum;                       /* Wf[^ۑpJE^ */

    /* Wf[^̕ۑ */
    /* saves the coordinate data */
    for (effectnum = EFFECTSIZE - 2;effectnum >= 0;effectnum --) {
        BallEffect[0][effectnum+1] = BallEffect[0][effectnum];
    }

    BallEffect[0][0].x = BallPos[0].x;
    BallEffect[0][0].y = BallPos[0].y;
    BallEffect[0][0].z = BallPos[0].z;

    /* c̕\ */
    /* draw the afterimages */
    for (effectnum = 0;effectnum < EFFECTSIZE;effectnum += 4) {
        njPushMatrix(NULL);

        /* ԂƂƂɎcȂ */
        /* if effectnum is bigger , alpha value is smaller */
        argb.a = 0.5F - (((float)effectnum / 2.0F) / EFFECTSIZE);

        /* }eA̐ݒi̐F͔j*/
        /* sets the material color */
        argb.r = 1.0F;
        argb.g = 1.0F;
        argb.b = 1.0F;
        njSetConstantMaterial(&argb);

        /* c̕` */
        /* draw the afterimage objects */
        njTranslate(NULL, BallEffect[0][effectnum].x,
                    BallEffect[0][effectnum].y, BallEffect[0][effectnum].z);
        njCnkDrawObject( object__sphere3_sphere3 );

        njPopMatrix(1);
    }
}

/*----------------------------------------------------------------------
         I
         exit function
----------------------------------------------------------------------*/
void njUserExit(void)
{
	njExitPrint();

    /* ̊J */
	/* frees the memory area */
    syFree(Matrix);
    syFree(VertexBuffer);

    /* VXȅI */
    /* exits the system    */
    sbExitSystem();
}
