/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F |Sfɓ蔻
  ҏW F 98-05-13 Jn
              98-06-19 SET5Ή
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}


/* O[oZNV */

/* O`̃ff[^ */
/* model data             */
extern NJS_OBJECT   G_ObjectBox[];
extern NJS_OBJECT   G_ObjectSphere[];
extern NJS_OBJECT   G_ObjectCapsule[];


/* fp\ */
/* model structure */

/*   */
/* sphere */
typedef struct {

    float r;
    NJS_SPHERE coli;

} MODEL_SPHERE;

/* {bNX */
/* box */
typedef struct {

    float   size;
    NJS_BOX coli;

} MODEL_BOX;

/* JvZ */
/* capsule  */
typedef struct {

    float r;
    Angle theta;
    Angle dt;
    NJS_POINT3  c1;
    NJS_POINT3  c2;
    NJS_CAPSULE coli;

} MODEL_CAPSULE;

/* fpp */
/* model structure(union) */
typedef union {

    MODEL_SPHERE  sphere;
    MODEL_BOX     box;
    MODEL_CAPSULE capsule;

} MY_MODEL;

/* IuWFNgp\ */
/* object structure */
enum TYPE {
       TYPE_SPHERE,
       TYPE_BOX,
       TYPE_CAPSULE,
       TYPE_END
};

typedef struct {

    enum TYPE type;

    NJS_POINT3 pos;
    NJS_POINT3 v;

    MY_MODEL   model;

} MY_OBJECT;

#define OBJECT_NUM  9
static MY_OBJECT G_Object[OBJECT_NUM];

/* }gNX֘A */
/* matrix  */
#define MATRIX_DEPTH 12
#define VERTEX_MAX   4096

static NJS_MATRIX     *G_Matrix;  /* }gNXX^bN */
static NJS_VERTEX_BUF *G_VBuf;  /* Zp_obt@ */

/*   _ */
/* light resource and view point */
static NJS_VIEW  G_View;
static NJS_LIGHT G_Light;

/* ֐̃vg^Cv */
/* prototype declarations */
void ObjectPhase(void);
void DrawObject(void);
void CollisionPhase(void);
int  CollisionObject(MY_OBJECT *obj1, MY_OBJECT *obj2);
void SetMaterial(float a, float r, float g, float b);
void CreateObject(int obj_num);
void CollisionEffect(MY_OBJECT *obj1, MY_OBJECT *obj2);
void CollisionEffectCapsule(MY_OBJECT *obj1, MY_OBJECT *obj2);
void PausePhase();
void ActivePhase();

/* RWp` */
/* for collision */
#define COLLISION_WEIGHT    40

/* |[YptO */
/* pause flag */
Bool g_bPause = FALSE;

/**********************************************************************

	njSetBorderColor( 0x00000000 );
    njUserInit --- NinjaVXe֐

**********************************************************************/

void njUserInit(void)
{
    /* VXȅ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

	/* msrbʂ̏c␳ */
	/* corrects the screen aspect ratio */
    njSetAspect(1.0F, 0.45F);
    
    /* ̊蓖 */
    /* allocates the memory */
    G_Matrix = (NJS_MATRIX *)syMalloc(sizeof (NJS_MATRIX) * MATRIX_DEPTH);
    G_VBuf = (NJS_VERTEX_BUF *)syMalloc(sizeof (NJS_VERTEX_BUF) * VERTEX_MAX);

    /* wiF̐ݒ */
	/* sets background color */
    njSetBackColor(ARGB(0xff, 0xc0, 0, 0), ARGB(0xff, 0, 0xc0, 0), 
												ARGB(0xff, 0, 0, 0xc0));
    /* _i[obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(VERTEX_MAX * 64 / 4, 0, VERTEX_MAX * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bN̏ */
	/* initializes the matrix stack */
    njInitMatrix(G_Matrix, MATRIX_DEPTH, 1);

    /* Zp_obt@̏ */
    /* initializes 3D system  */ 
    njInit3D(G_VBuf, VERTEX_MAX);

    /* ̏ */
    /* initializes the light source */
    njCreateLight(&G_Light, NJD_LAMBERT_DIR);

    /* r[̏ */
    /* initializes the user-defined view */
    njInitView(&G_View);

    /* r[̐ݒ */
    /* sets the user-defined view as the current view */
    njSetView(&G_View);

    /* }eAύXł悤ɐݒ */
    /* the material is changeable */
    njControl3D(NJD_CONTROL_3D_CONSTANT_MATERIAL);

    /* IuWFNg̐ */
    /* creates the object */
    CreateObject(OBJECT_NUM);

	/* eLXg̐Fw */
	/* sets the color of debug printing */
	njPrintColor(ARGB(0xff, 0xc0, 0xc0, 0xc0));
}


/**********************************************************************

    CreateObject --- IuWFNg𐶐

**********************************************************************/

void CreateObject(int obj_num)
{
    NJS_POINT3 pos;
    int counter;

    /* IuWFNgf[^̏ */
    /* initializes object's data  */
    pos.x = -8.0F;
    pos.y =  8.0F;
    for (counter = 0; counter < obj_num; counter++) {
        G_Object[counter].type = counter % TYPE_END;
        switch(G_Object[counter].type) {
        case TYPE_SPHERE:
            /* f[^̏ */
            /* initializes spheres data */
            G_Object[counter].type = TYPE_SPHERE;
            G_Object[counter].pos.x =  pos.x;
            G_Object[counter].pos.y =  pos.y;
            G_Object[counter].pos.z = -20.0F;
            G_Object[counter].v.x = -0.2F;
            G_Object[counter].v.y =  0.2F;
            G_Object[counter].model.sphere.r = 1.5F;
            break;

        case TYPE_BOX:
            /* {bNXf[^̏ */
            /* initializes boxs data */
            G_Object[counter].type = TYPE_BOX;
            G_Object[counter].pos.x = pos.x;
            G_Object[counter].pos.y = pos.y;
            G_Object[counter].pos.z = -20.0F;
            G_Object[counter].v.x = 0.1F;
            G_Object[counter].v.y = 0.1F;
            G_Object[counter].model.box.size = 2.0F;
            break;

        case TYPE_CAPSULE:
            /* JvZf[^̏ */
            /* initializes cupsules data */
            G_Object[counter].type = TYPE_CAPSULE;
            G_Object[counter].pos.x = pos.x;
            G_Object[counter].pos.y = pos.y;
            G_Object[counter].pos.z =-20.0F;
            G_Object[counter].v.x = 0.05F;
            G_Object[counter].v.y = 0.05F;
            G_Object[counter].model.capsule.r = 1.0F;
            G_Object[counter].model.capsule.theta = NJM_DEG_ANG(0.0F);
            G_Object[counter].model.capsule.dt = NJM_DEG_ANG(2.0F);
            G_Object[counter].model.capsule.c1.x = 0.0F;
            G_Object[counter].model.capsule.c1.y = 2.0F;
            G_Object[counter].model.capsule.c1.z = 0.0F;
            G_Object[counter].model.capsule.c2.x = 0.0F;
            G_Object[counter].model.capsule.c2.y =-2.0F;
            G_Object[counter].model.capsule.c2.z = 0.0F;
            break;
        }
        pos.x += 8.0F;
        if (pos.x > 10.0F) {
            pos.x = -8.0F;
            pos.y -= 8.0F;
        }
    }
}


/**********************************************************************

    njUserMain --- NinjaVXeC֐

**********************************************************************/

Sint32 njUserMain(void)
{
    NJS_PERIPHERAL *per;

    /* WCXeBbN|[gP̃ytF擾 */
    /* gets the peripheral information (port A) */
    per = njGetPeripheral(NJD_PORT_A0);    

    /* {^ꂽǂ`FbN */
    /* checks the pressing start button */
    if (per->press & NJD_DGT_ST) {
        g_bPause = !g_bPause;
    }

	/* |[YԂɂď؂ւ */
	/* pause or not */
	if (!g_bPause) {
	    njLightAllOn();
		ActivePhase();
	} else {
	    njLightAllOff();
	    PausePhase();
	}


    /* fobO̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 4), "Ninja Sample COLLISION");

    return NJD_USER_CONTINUE;
}


/**********************************************************************

    ActivePhase --- ANeBu֐

**********************************************************************/

void ActivePhase(void)
{
    /* }gNXX^bÑNA */
    /* clears the matrix stack    */
    njClearMatrix();

    /* IuWFNg̏ */
    /* moves objects      */
    ObjectPhase();

    /* RW */
    /* checks the collision */
    CollisionPhase();

    /* IuWFNg̕` */
    /* draws the objects  */
    DrawObject();
}


/**********************************************************************

    PausePhase --- |[Y֐

**********************************************************************/

void PausePhase(void) 
{
    /* }gNXX^bÑNA */
    /* clears the matrix stack */
    njClearMatrix();

    /* IuWFNg̕` */
    /* draws objects */
    DrawObject();

	/* |[Ŷm点 */
	/* prints 'PAUSE' message */ 
    njPrintC(NJM_LOCATION(38, 30), "PAUSE");
}


/**********************************************************************

    ObjectPhase --- IuWFNg̏

    argument: Ȃ

**********************************************************************/

void ObjectPhase(void)
{
    MY_OBJECT *obj = G_Object;
    NJS_MATRIX mat;
    NJS_POINT3 c1, c2;
    int counter;

    for (counter = 0; counter < OBJECT_NUM; counter++) {

        /* IuWFNg̈ړ */
        /* moves the objects      */
        obj->pos.x += obj->v.x;
        if (obj->pos.x < -10.0F || obj->pos.x > 10.0F) {
            obj->v.x = -obj->v.x;
            obj->pos.x += obj->v.x * 2;
        }
        obj->pos.y += obj->v.y;
        if (obj->pos.y < -10.0F || obj->pos.y > 10.0F) {
            obj->v.y = -obj->v.y;
            obj->pos.y += obj->v.y * 2;
        }

        /* JvZȂ] */
        /* rotates the objects, if the type is capsule */
        if (obj->type == TYPE_CAPSULE) {
            obj->model.capsule.theta += obj->model.capsule.dt;
        }

        /* RWpf[^̍쐬 */
        /* makes the collision data */
        switch(obj->type) {

        case TYPE_SPHERE:
            obj->model.sphere.coli.c = obj->pos;
            obj->model.sphere.coli.r = obj->model.sphere.r;
            break;

        case TYPE_CAPSULE:
            njUnitMatrix(&mat);
            njTranslate(&mat, obj->pos.x, obj->pos.y, obj->pos.z);
            njRotateZ(&mat, obj->model.capsule.theta);
            njCalcPoint(&mat, &(obj->model.capsule.c1), &c1);
            njCalcPoint(&mat, &(obj->model.capsule.c2), &c2);
            obj->model.capsule.coli.c1 = c1;
            obj->model.capsule.coli.c2 = c2;
            obj->model.capsule.coli.r  = obj->model.capsule.r;
            break;
        
        case TYPE_BOX:
            obj->model.box.coli.v[0].x = obj->pos.x - obj->model.box.size;
            obj->model.box.coli.v[0].y = obj->pos.y + obj->model.box.size;
            obj->model.box.coli.v[0].z = obj->pos.z + obj->model.box.size;

            obj->model.box.coli.v[1].x = obj->pos.x - obj->model.box.size;
            obj->model.box.coli.v[1].y = obj->pos.y + obj->model.box.size;
            obj->model.box.coli.v[1].z = obj->pos.z - obj->model.box.size;

            obj->model.box.coli.v[2].x = obj->pos.x + obj->model.box.size;
            obj->model.box.coli.v[2].y = obj->pos.y + obj->model.box.size;
            obj->model.box.coli.v[2].z = obj->pos.z - obj->model.box.size;

            obj->model.box.coli.v[3].x = obj->pos.x + obj->model.box.size;
            obj->model.box.coli.v[3].y = obj->pos.y + obj->model.box.size;
            obj->model.box.coli.v[3].z = obj->pos.z + obj->model.box.size;

            obj->model.box.coli.v[4].x = obj->pos.x - obj->model.box.size;
            obj->model.box.coli.v[4].y = obj->pos.y - obj->model.box.size;
            obj->model.box.coli.v[4].z = obj->pos.z + obj->model.box.size;

            obj->model.box.coli.v[5].x = obj->pos.x - obj->model.box.size;
            obj->model.box.coli.v[5].y = obj->pos.y - obj->model.box.size;
            obj->model.box.coli.v[5].z = obj->pos.z - obj->model.box.size;

            obj->model.box.coli.v[6].x = obj->pos.x + obj->model.box.size;
            obj->model.box.coli.v[6].y = obj->pos.y - obj->model.box.size;
            obj->model.box.coli.v[6].z = obj->pos.z - obj->model.box.size;
            
            obj->model.box.coli.v[7].x = obj->pos.x + obj->model.box.size;
            obj->model.box.coli.v[7].y = obj->pos.y - obj->model.box.size;
            obj->model.box.coli.v[7].z = obj->pos.z + obj->model.box.size;
            break;
        }
        obj++;
    }
}


/**********************************************************************

    DrawObject --- IuWFNg̕`揈

    argument: Ȃ

**********************************************************************/

void DrawObject(void)
{
    MY_OBJECT *obj = G_Object;
    int counter;

    for (counter = 0; counter < OBJECT_NUM; counter++) {
        switch(obj->type) {
        case TYPE_SPHERE:
            SetMaterial(1.0F, 1.0F, 0.0F, 0.0F);
            njPushMatrix(NULL);
                njTranslate(NULL, obj->pos.x, obj->pos.y, obj->pos.z);
                njScale(NULL, 0.1F, 0.1F, 0.1F);
                njDrawObject(G_ObjectSphere);
            njPopMatrix(1);
            break;

        case TYPE_BOX:
            SetMaterial(1.0F, 0.0F, 1.0F, 0.0F);
            njPushMatrix(NULL);
                njTranslate(NULL, obj->pos.x, obj->pos.y, obj->pos.z);
                njScale(NULL, 2.0F, 2.0F, 2.0F);
                njDrawObject(G_ObjectBox);
            njPopMatrix(1);
            break;

        case TYPE_CAPSULE:
            SetMaterial(1.0F, 0.0F, 0.0F, 1.0F);
            njPushMatrix(NULL);
                njTranslate(NULL, obj->pos.x, obj->pos.y, obj->pos.z);
                njRotateZ(NULL, obj->model.capsule.theta);
                njScale(NULL, 0.5F, 0.5F, 0.5F);
                njDrawObject(G_ObjectCapsule);
            njPopMatrix(1);
            break;
        }
        obj++;
    }
}


/**********************************************************************

    SetMaterial --- }eAݒ

    argument: float aij
              float riԐj
              float giΐj
              float bij

**********************************************************************/

void SetMaterial(float a, float r, float g, float b)
{
    NJS_ARGB argb;

    /* }eA̐ݒ */
    /* sets the material */
    argb.a = a;
    argb.r = r;
    argb.g = g;
    argb.b = b;
    njSetConstantMaterial(&argb);
}


/**********************************************************************

    CollisionPhase --- IuWFNg̓蔻菈

    argument: Ȃ

**********************************************************************/

void CollisionPhase(void)
{
    int cnt_src, cnt_dest;

    /* SẴIuWFNgɂē蔻s */
    /* checks the collision for all objects         */
    for (cnt_src = 0; cnt_src < OBJECT_NUM; cnt_src++) {
        for (cnt_dest = cnt_src + 1; cnt_dest < OBJECT_NUM; cnt_dest++) {
            if (CollisionObject(&G_Object[cnt_src], &G_Object[cnt_dest])) {
                CollisionEffect(&G_Object[cnt_src], &G_Object[cnt_dest]);
            }
        }
    }
}


/**********************************************************************

    CollisionObject --- IuWFNg̓蔻菈

    argument: MY_OBJECT *obj1iIuWFNgPj
              MY_OBJECT *obj2iIuWFNgQj

**********************************************************************/

int CollisionObject (MY_OBJECT *obj1, MY_OBJECT *obj2)
{
    switch(obj1->type) {
    case TYPE_SPHERE:
        if (obj2->type == TYPE_SPHERE) {
            return njCollisionCheckSS(&(obj1->model.sphere.coli),
                                   &(obj2->model.sphere.coli));
        } else if (obj2->type == TYPE_BOX) {
            return njCollisionCheckBS(&(obj2->model.box.coli),
                                   &(obj1->model.sphere.coli));
        } else if (obj2->type == TYPE_CAPSULE) {
            return njCollisionCheckSC(&(obj1->model.sphere.coli),
                                   &(obj2->model.capsule.coli));
        }
        break;

    case TYPE_BOX:
        if (obj2->type == TYPE_SPHERE) {
            return njCollisionCheckBS(&(obj1->model.box.coli),
                                   &(obj2->model.sphere.coli));
        } else if (obj2->type == TYPE_BOX) {
            return njCollisionCheckBB(&(obj1->model.box.coli),
                                   &(obj2->model.box.coli));
        } else if (obj2->type == TYPE_CAPSULE) {
            return njCollisionCheckBC(&(obj1->model.box.coli),
                                   &(obj2->model.capsule.coli));
        }
        break;

    case TYPE_CAPSULE:
        if (obj2->type == TYPE_SPHERE) {
            return njCollisionCheckSC(&(obj2->model.sphere.coli),
                                   &(obj1->model.capsule.coli));
        } else if (obj2->type == TYPE_BOX) {
            return njCollisionCheckBC(&(obj2->model.box.coli),
                                   &(obj1->model.capsule.coli));
        } else if (obj2->type == TYPE_CAPSULE) {
            return njCollisionCheckCC(&(obj1->model.capsule.coli),
                                   &(obj2->model.capsule.coli));
        }
        break;
    }
}


/**********************************************************************

    CollisionEffect --- 蔻ɂ鏈

    argument: MY_OBJECT *objiIuWFNgւ̃|C^j

**********************************************************************/

void CollisionEffect(MY_OBJECT *obj1, MY_OBJECT *obj2)
{
    obj1->v.x = (obj1->pos.x - obj2->pos.x) / COLLISION_WEIGHT;
    obj1->v.y = (obj1->pos.y - obj2->pos.y) / COLLISION_WEIGHT;
    obj1->v.z = (obj1->pos.z - obj2->pos.z) / COLLISION_WEIGHT;
    obj2->v.x = (obj2->pos.x - obj1->pos.x) / COLLISION_WEIGHT;
    obj2->v.y = (obj2->pos.y - obj1->pos.y) / COLLISION_WEIGHT;
    obj2->v.z = (obj2->pos.z - obj1->pos.z) / COLLISION_WEIGHT;

    obj1->pos.x += obj1->v.x;
    obj1->pos.y += obj1->v.y;
    obj2->pos.x += obj2->v.x;
    obj2->pos.y += obj2->v.y;
}


/**********************************************************************

    njUserExit --- NinjaVXeI֐

**********************************************************************/

void njUserExit(void)
{
	njExitPrint();
	
    /* ̊J */
  	/* frees the memory area */
    syFree(G_Matrix);
    syFree(G_VBuf);
    
    /* VXȅI܂ ŌɕKĂяoĂ */
    /* exits the system    */
    sbExitSystem();
}
