
/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD

  ^Cg F s̐ݒA
              O[AtbgVF[fBO̐؂ւ
              parallel light,shading mode
  ҏW F 98-05-01 쐬Jn
              98-06-19 SET5Ή
              98-07-08 C0ĂtɑΉi|[Yǉj
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 10
#define MAX_VERTEXES 10000

extern NJS_CNK_OBJECT object_segalogo_s1_s1[];

static NJS_MATRIX *Matrix;
static NJS_VERTEX_BUF *VBuf;

static NJS_VIEW View;
static NJS_LIGHT Light;

PDS_PERIPHERAL* Per;

void PauseMode(void);

/************************************************************
  ֐
  initialize
************************************************************/
void njUserInit(void)
{
    /* VXe */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* ʂ̃AXyNg␳ */
	/* corrects the screen aspect ratio */
    njSetAspect(1.0F, 0.45F);

    /* obt@pAP[g */
    /* allocates the memory */
    Matrix = syMalloc(sizeof(NJS_MATRIX) * MAX_MATRIX_DEPTH);
    VBuf = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* obNOEhJ[ݒ */
	/* sets background color */
    njSetBackColor(ARGB(0, 128, 0, 0),
        ARGB(0, 128, 0, 0), ARGB(0, 128, 128, 0));

    /* _f[^o^pobt@AP[giP̒_USoCg^Sj */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bNiPCgZp̃}gNXgpj */
  	/* initializes the matrix stack (1:for the light)*/
    njInitMatrix(Matrix, MAX_MATRIX_DEPTH, 1);

    /* RcVXȅif`̂߂̃[Nobt@j */
    /* initializes 3D system  */ 
    njInit3D(VBuf, MAX_VERTEXES);

    /* ̎ނ߃CgVɐݒuiso^j
      iʒu0,0,0ji0,0,-1j */
    /* initializes the light source  (Parallel light source ) */
    /* position( 0,0,0)/ direction (0,0,-1) */
    njCreateLight(&Light, NJD_SPEC_DIR);

    /* r[̏iʒu0,0,0ji0,0,-1jiXOxj */
    /* initializes the user-defined view */
    /* position(0,0,0) / direction(0,0,-1) / ( sight tilt = 0)*/
    njInitView(&View);

    /* [U̗pӂr[Jgr[Ɏw */
	/* specifies the current view */
    njSetView(&View);

    /* fobO̐F */
    /* sets the text color */
    njPrintColor(ARGB(255, 150, 150, 150));
}

/************************************************************
  s郁C
  main
************************************************************/
Sint32 njUserMain(void)
{
    static int timer = 0; /* VF[fBO؂ւ^C}[ */
                          /* timer for changing the shading mode */
    static enum {
        SHADING_FLAT,
        SHADING_GOURAUD,
        SHADING_END
    } shading = SHADING_FLAT; /* VF[fBO[h */
                              /* shading mode */

    static Angle angleX = NJM_DEG_ANG(0);
    static Angle angleY = NJM_DEG_ANG(0);
    static Angle angleZ = NJM_DEG_ANG(0);

    static Bool pauseMode = FALSE;

    /* Tv̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(4, 2), "Ninja Sample LIGHT");
    njPrintC(NJM_LOCATION(4, 4), "Press Start Button To Pause");

    /* [h̕\ */
    /* prints the shading mode */
    if (shading == SHADING_FLAT) {
        njPrintC(NJM_LOCATION(4, 6), "Mode = Flat Shading");
    }
    else {
        njPrintC(NJM_LOCATION(4, 6), "Mode = Gouraud Shading");
    }

    /* ytF`FbN */
    /* gets the peripheral information */
    Per = pdGetPeripheral(PDD_PORT_A0);
    if (Per->press & PDD_DGT_ST) { /* X^[g{^ꂽ */
                                   /* pressing the start button ? */
        pauseMode = !pauseMode;
    }

    /* |[Yʂ */
    /* puase */
    if (pauseMode == TRUE) {
        NJS_ARGB argb = { -0.8F, 0.0F, 0.0F, 0.0F };
        njControl3D(NJD_CONTROL_3D_CNK_CONSTANT_ATTR |
                    NJD_CONTROL_3D_OFFSET_MATERIAL);
        if (shading == SHADING_FLAT) {
            njSetConstantAttr(0xffffffff, NJD_FST_FL |
                                          NJD_FST_UA);
        }
        else {
            njSetConstantAttr(0xffffffff, NJD_FST_UA);
        }
        njSetConstantMaterial(&argb);
        PauseMode();
    }
    else {
        njControl3D(NJD_CONTROL_3D_CNK_CONSTANT_ATTR);
        if (shading == SHADING_FLAT) {
            njSetConstantAttr(0xffffffff, NJD_FST_FL);
        }
        else {
            njSetConstantAttr(0xffffffff, 0);
        }
    }

    if (pauseMode == FALSE) {
        /* QboVF[fBO؂ւ */
        /* changes the shading mode every 2 seconds */
        timer++;
        if (timer == 120) {
            if (shading == SHADING_FLAT) {
                shading = SHADING_GOURAUD;
            }
            else {
                shading = SHADING_FLAT;
            }
            timer = 0;
        }
        /* ]p̍XV */
        /* updates the rotate angle */
        angleX += NJM_DEG_ANG(0.5F);
        angleY += NJM_DEG_ANG(0.5F);
        angleZ += NJM_DEG_ANG(0.5F);
    }

    njClearMatrix();
    njTranslate(NULL, 0.0F, 0.0F, -7.5F);
    njRotateXYZ(NULL, angleX, angleY, angleZ);
    njCnkDrawObject(object_segalogo_s1_s1);

    return NJD_USER_CONTINUE;
}

/************************************************************
  sIɌĂяoI
  exit
************************************************************/
void njUserExit(void)
{
	njExitPrint();

    /*  */
	/* frees the memory area */
    syFree(Matrix);
    syFree(VBuf);

    /* VXȅI */
    /* exits the system    */
    sbExitSystem();
}

/************************************************************
  |[YɌĂяo鏈
  pause
************************************************************/
void PauseMode(void)
{
    static Sint16 interval = 0;
    static Bool display = TRUE;

    interval++;
    if (interval >= 30) {
        display = !display;
        interval = 0;
    }
    if (display == TRUE) {
        njPrintSize(32);
        njPrintC(NJM_LOCATION(7, 6),"PAUSE!");
        njPrintSize(8);
    }
}
