
/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD

  ^Cg F }gNX̃vbV|bv
              push and pop the matrix stack
  ҏW F 98-05-08 쐬Jn
              98-06-19 SET5Ή
              98-07-08 C0ĂtɑΉi|[Yǉj
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 10
#define MAX_VERTEXES 10000

extern NJS_CNK_OBJECT object_cube_root[];

static NJS_MATRIX *Matrix;
static NJS_VERTEX_BUF *VBuf;
static NJS_VIEW View;
static NJS_LIGHT Light;

/* O[v̐ݒ */
/* the number of groups */
#define GROUP_NUM 4

/* P̃O[v̐̕ݒ */
/* the number of the object in a group */
#define OBJECT_NUM 6

/* Ŝ̉]xݒ */
/* speed of whole rotation */
#define ROTATION_SPEED_ALL 1.0F

/* O[v̉]x͈̔͂ݒ */
/* maximum rotation speed of a group */
#define MAX_SPEED_GROUP 4.0F

/* ̂̉]x͈̔͂ݒ */
/* maximum rotation speed of a object */
#define MAX_SPEED_OBJECT 8.0F

/* ]Ԃ\ */
/* rotation object */
typedef struct {
    Angle ang; /* px  */
               /* angle */
    Angle rsp; /* ]̑ */
               /* speed */
} RotateState;

static RotateState Group[GROUP_NUM]; /* O[v̉] */
                                     /* rotation status of a group  */
static RotateState Object[GROUP_NUM][OBJECT_NUM]; /* ̂̉] */
                                     /* rotation status of a object */

PDS_PERIPHERAL* Per;

void PauseMode(void);

/************************************************************
  ֐
  initialize
************************************************************/
void njUserInit(void)
{
    /* VXe */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* ʂ̃AXyNg␳ */
	/* corrects the screen aspect ratio */
    njSetAspect( 1.0F, 0.45F );

    /* obt@pAP[g */
    /* allocates the memory */
    Matrix = syMalloc(sizeof(NJS_MATRIX) * MAX_MATRIX_DEPTH);
    VBuf = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* obNOEhJ[ݒ */
	/* sets background color */
    njSetBackColor(ARGB(0, 0, 0, 64), ARGB(0, 0, 0, 64), ARGB(0, 64, 0, 0));

    /* _f[^o^pobt@AP[giP̒_US^SoCgj */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bNiPCgZp̃}gNXgpj */
  	/* initializes the matrix stack (1:for the light)*/
    njInitMatrix(Matrix, MAX_MATRIX_DEPTH, 1);

    /* RcVXȅif`̂߂̃[Nobt@j */
    /* initializes 3D system  */ 
    njInit3D(VBuf, MAX_VERTEXES);

    /* ̎ނ߃CgVɐݒuiso^j
      iʒu0,0,0ji0,0,-1j */
    /* initializes the light source  (Parallel light source ) */
    /* position( 0,0,0)/ direction (0,0,-1) */
    njCreateLight(&Light, NJD_SPEC_DIR);

    /* r[̏iʒu0,0,0ji0,0,-1jiXOxj */
    /* initializes the user-defined view */
    /* position(0,0,0) / direction(0,0,-1) / ( sight tilt = 0)*/
    njInitView(&View);

    /* [U̗pӂr[Jgr[Ɏw */
	/* specifies the current view */
    njSetView(&View);

    /* fobO̐F */
    /* sets the text color */
    njPrintColor(ARGB(255, 150, 150, 150));

    /* ̎ݒ */
    /* set the randam seed */
    njRandomSeed(0);

    {
        int grp; /* O[vɑ΂郋[vJE^ */
                 /* counter of groups */
        int obj; /* ̂ɑ΂郋[vJE^ */
                 /* counter of object */

        /* O[vƂ̏pxƉ]̑ݒ */
        /* sets the initial angle and speed to each group */
        for (grp = 0; grp < GROUP_NUM; grp++) {
            RotateState *grpp = &Group[grp];
            grpp->ang = NJM_DEG_ANG(0.0F);
            grpp->rsp = (Angle)(NJM_DEG_ANG(1.0F) * (njRandom() *
                MAX_SPEED_GROUP * 2.0F - MAX_SPEED_GROUP));
        }

        /* ̂Ƃ̏pxƉ]̑ݒ */
        /* sets the initial angle and speed to each object */
        for (grp = 0; grp < GROUP_NUM; grp++) {
            for (obj = 0; obj < OBJECT_NUM; obj++) {
                RotateState *objp = &Object[grp][obj];
                objp->ang = NJM_DEG_ANG(0.0F);
                objp->rsp = (Angle)(NJM_DEG_ANG(1.0F) * (njRandom() *
                    MAX_SPEED_OBJECT * 2.0F - MAX_SPEED_OBJECT));
            }
        }
    }

    /* Ŝ̉]̒S */
    /* move to the center of rotation */
    njTranslate(NULL, 0.0F, -5.0F, -30.0F);
}

/************************************************************
  s郁C
************************************************************/
Sint32 njUserMain(void)
{
    static Angle angAll = NJM_DEG_ANG(0.0F); /* Ŝ̉]p*/
                                             /* rotation angle of the whole objects */

    static Bool pauseMode = FALSE;

    /* Tv̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(4, 2), "Ninja Sample MATRIX STACK");
    njPrintC(NJM_LOCATION(4, 4), "Press Start Button To Pause");

    /* ytF`FbN */
    /* gets the peripheral information */
    Per = pdGetPeripheral(PDD_PORT_A0);
    if (Per->press & PDD_DGT_ST) { /* X^[g{^ꂽ */
                                   /* pressing the start button ? */
        pauseMode = !pauseMode;
    }

    /* |[Yʂ */
    /* pause */
    if (pauseMode == TRUE) {
        njLightOff(&Light);
        PauseMode();
    }
    else {
        njLightOn(&Light);
    }

    { /* IuWFNg\ */
      /* draws objects */
        int grp; /* O[vɑ΂郋[vJE^ */
                 /* counter of group  */
        int obj; /* ̂ɑ΂郋[vJE^ */
                 /* counter of object */

        for (grp = 0; grp < GROUP_NUM; grp++) {
            njPushMatrix(NULL);
            /* O[v̉]̒S */
            /* moves to center of the group */
            njRotateY(NULL, angAll + (Angle)(NJM_DEG_ANG(1.0F) *
                grp * (360.0F / GROUP_NUM)));
            njTranslate(NULL, 0.0F, 0.0F, 10.0F);

            for (obj = 0; obj < OBJECT_NUM; obj++) {
                njPushMatrix(NULL);
                /* ̂̉]̒S */
                /* moves to center of the object */
                njRotateY(NULL, Group[grp].ang + (Angle)(NJM_DEG_ANG(1.0F) *
                    obj * (360.0F / OBJECT_NUM)));
                njTranslate(NULL, 0.0F, 0.0F, 4.0F);
                njRotateY(NULL, Object[grp][obj].ang);
                
                /* ̂\ */
                /* draws the object */
                njCnkDrawObject(object_cube_root);
                njPopMatrix(1);
            }
            njPopMatrix(1);
        }
    }

    if (pauseMode == FALSE) {
        /* Ŝ̉]pXV */
        /* updates the angle of whole objects */
        angAll += NJM_DEG_ANG(ROTATION_SPEED_ALL);

        {
            int grp; /* O[vɑ΂郋[vJE^ */
                     /* counter of group  */
            int obj; /* ̂ɑ΂郋[vJE^ */
                     /* counter of object */
                     
            /* O[v̉]pXV */
            /* updates the angle of groups */
            for (grp = 0; grp < GROUP_NUM; grp++) {
                RotateState *grpp = &Group[grp];
                grpp->ang += grpp->rsp;
            }
            /* ̂̉]pXV */
            /* updates the angle of objects */
            for (grp = 0; grp < GROUP_NUM; grp++) {
                for (obj = 0; obj < OBJECT_NUM; obj++) {
                    RotateState *objp = &Object[grp][obj];
                    objp->ang += objp->rsp;
                }
            }
        }
    }

    return NJD_USER_CONTINUE;
}

/************************************************************
  sIɌĂяoI
  exit
************************************************************/
void njUserExit(void)
{
	njExitPrint();

    /*  */
	/* frees the memory area */
    syFree(Matrix);
    syFree(VBuf);

    /* VXȅI */
    /* exits the system    */
    sbExitSystem();
}

/************************************************************
  |[YɌĂяo鏈
  pause
************************************************************/
void PauseMode(void)
{
    static Sint16 interval = 0;
    static Bool display = TRUE;

    interval++;
    if (interval >= 30) {
        display = !display;
        interval = 0;
    }
    if (display == TRUE) {
        njPrintSize(32);
        njPrintC(NJM_LOCATION(7, 6),"PAUSE!");
        njPrintSize(8);
    }
}
