/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD

  ^Cg F |S]
              rotate the polygon
  ҏW F 98-04-30 쐬Jn
              98-06-19 SET5Ή
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 10
#define MAX_VERTEXES 200

#define ROTATE_ANGLE_SPEED NJM_DEG_ANG(1.0F)
#define POINT_NUMBER 3/* _̐ */
                      /* the number of vertex */
#define SCREEN_SAVE_TIME 30

static NJS_POINT3COL Point;
static NJS_POINT3 PointData[POINT_NUMBER];
static NJS_COLOR ColorData[POINT_NUMBER];

static NJS_MATRIX *TranslationMatrix;
static NJS_VERTEX_BUF *VertexBuffer;

static NJS_VIEW View;
static NJS_LIGHT DirLight;

/*--------------------------------------------
    
    initialize
--------------------------------------------*/
void njUserInit(void)
{
    /* VXȅ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x240_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* ʂ̃AXyNg␳ */
	/* corrects the screen aspect ratio */
    njSetAspect( 1.0F, 0.45F );

    /* ̊蓖 */
    /* allocates the memory */
    TranslationMatrix = syMalloc(sizeof(NJS_MATRIX) * MAX_MATRIX_DEPTH);
    VertexBuffer   = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* wiF̐ݒ */
	/* sets background color */
    njSetBackColor(0x00000000, 0x00000000, 0x00000000);

    /* _i[obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gbNXX^bN̏ */
  	/* initializes the matrix stack */
    njInitMatrix(TranslationMatrix, MAX_MATRIX_DEPTH, 1);

    /* 3DVXȅ */
    /* initializes 3D system  */ 
    njInit3D(VertexBuffer, MAX_VERTEXES);

    /* ̏ */
    /* initializes the light source */
    njCreateLight(&DirLight, NJD_DIR_LIGHT);

    /* _̏ */
    /* initializes the user-defined view */
    njInitView(&View);

    /* _̐ݒ */
	/* specifies the current view */
    njSetView(&View);

    /* }gbNXX^bÑNA */
	/* clears the matrix stack */
    njClearMatrix();    

    /* _WAF̐ݒ */
    /* sets the potision and color */
    PointData[0].x = 0.0F;
    PointData[0].y = 1.15F;
    PointData[0].z = 0.0F;
    ColorData[0].color = 0xFFC0C0C0;

    PointData[1].x = -1.0F;
    PointData[1].y = -0.58F;
    PointData[1].z = 0.0F;
    ColorData[1].color = 0xFFC0C0C0;

    PointData[2].x = 1.0F;
    PointData[2].y = -0.58F;
    PointData[2].z = 0.0F;
    ColorData[2].color = 0xFFC0C0C0;

    Point.p = PointData;
    Point.col = ColorData;
    Point.tex = NULL;
    Point.num = POINT_NUMBER;

	/* F̐ݒ */
    /* sets the text color */
	njPrintColor(0xFFC0C0C0);
}

Sint32 njUserMain(void)
{
    static Angle rotateAng = NJM_DEG_ANG(0.0F);/* ]px */
                                               /* rotation angle */
	static int counter = 0;
	static PDS_PERIPHERAL *key;/* {^ */
	                           /* peripheral information */
	static Bool pause = FALSE; /* |[YĂ邩ǂ */
                               /* pause flag */
	/* |[Y̔ */
	/* checks pause */
	key = pdGetPeripheral(PDD_PORT_A0);

	if (key->press & PDD_DGT_ST) {
	    if (pause == FALSE) {
		    pause = TRUE;
		} else {
		    pause = FALSE;
		}
	}

    /* ^Cg\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 4), "Ninja Sample POLYGON ROTATION");

    /* }gNX̑Ҕ */
	/* pushes the matrix stack */
    njPushMatrix(NULL);
    
    /* `_ւ̈ړ */
    /* moves to the draw point */
    njTranslate(NULL, 0.0F, 0.0F, -10.0F);

    /* ] */
    /* rotates */
    njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);
    
    /* Op`̕` */
    /* draws triangle */
	if (pause == FALSE) {
	    njDrawTriangle3D(&Point, 1, NJD_DRAW_NORMAL|NJD_FILL);
	} else {
	    counter++;
	    if (counter > SCREEN_SAVE_TIME) {
		    njDrawTriangle3D(&Point, 1, NJD_DRAW_NORMAL|NJD_FILL);
			njPrintC(NJM_LOCATION(30, 5), "PAUSE");
		}
		if (counter == SCREEN_SAVE_TIME * 2) {
				counter = 0;
		}
	}
    /* ]px̐ݒ */
    /* sets the rotation angle */
	if (pause == FALSE) {
	    rotateAng += ROTATE_ANGLE_SPEED;
	}
    /* }gNX̕A */
    /* pops the matrix stack */
    njPopMatrix(1);
    return NJD_USER_CONTINUE;
}
/*-----------------------------------
    I
    exit
-----------------------------------*/
void njUserExit(void)
{
	njExitPrint();

    /* ̊J */
	/* frees the memory area */
    syFree(TranslationMatrix);
    syFree(VertexBuffer);

    sbExitSystem();
}
