/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F ̂ɉeiVhE{Eł͖j
              shadow ( but it's not shadow volume )
  ҏW F 98-04-28 Jn
              98-06-19 SET5Ή
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}


/* O`ff[^ */
/* external declaration of model data */
extern NJS_OBJECT object_logo[];
extern NJS_OBJECT object_arrow[];

/* O[of[^ZNV */
/* global data */
#define VERTEX_MAX      4096
#define MATRIX_DEPTH    12

/* 3D֘Af[^ */
/* 3D data */
static NJS_MATRIX     *g_Matrix;
static NJS_VERTEX_BUF *g_VBuf;

static NJS_VIEW    g_View;
static NJS_LIGHT   g_Light;

/* ̃f[^ */
/* light source */
static NJS_POINT3  g_LightDir;
static NJS_POINT3  g_LightPos;
static Angle       g_LightTheta;
static Angle       g_LightPhai;
static float       g_LightR;

/* nʂ̕` */
static float g_GroundHeight = -3.0F;

/* ֐̃vg^Cv */
/* prototype declaration */
void DrawPlane(float size);
void LightPhase();
void ObjectPhase();
void ShadowPhase();

/* |[YtO */
/* pause flag       */
Bool g_bPause = FALSE;

/* |[Y֐ */
/* pause function   */
void PausePhase(void);

/* ANeBu֐ */
/* active (not pause) function */
void ActivePhase(void);

/**************************************************************************

	njSetBorderColor( 0x00000000 );
    njUserInit --- NinjaVXe֐

**************************************************************************/

void njUserInit(void)
{
    /* ʃ[h̐ݒ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem( NJD_RESOLUTION_640x480_NTSCI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* ̊蓖 */
    /* allocates the memory */
    g_Matrix = (NJS_MATRIX *)syMalloc(sizeof(NJS_MATRIX) * MATRIX_DEPTH);
    g_VBuf = (NJS_VERTEX_BUF *)syMalloc(sizeof(NJS_VERTEX_BUF) * VERTEX_MAX);

    /* _obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(VERTEX_MAX * 64 / 4, 0, VERTEX_MAX * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* fr[}gNX */
  	/* initializes the matrix stack */
    njInitMatrix(g_Matrix, MATRIX_DEPTH, 1);

    /* Zp_obt@̊m */
    /* initializes 3D system  */ 
    njInit3D(g_VBuf, VERTEX_MAX);

    /* r[̏Ɛݒ */
    /* initializes the user-defined view */
    njInitView(&g_View);
    njSetView(&g_View);

    /* Cg̏ */
    /* initializes the light source */
    g_LightTheta = NJM_DEG_ANG(45.0F);
    g_LightPhai  = NJM_DEG_ANG(45.0F);
    g_LightR = 5.0F;
    g_LightDir.x = -1.0F;
    g_LightDir.y = -1.0F;
    g_LightDir.z = -1.0F;
    njCreateLight(&g_Light, NJD_DIR_LIGHT);
    njSetLightDirection(&g_Light, g_LightDir.x, g_LightDir.y, g_LightDir.z);

    /* }eAύXł悤ɐݒ */
    /* model material data changed to a fixed value */
    njControl3D(NJD_CONTROL_3D_CONSTANT_MATERIAL);

    /* obNOEh̃J[ݒ */
  	/* sets background color */
    njSetBackColor(ARGB(0xff, 0xc0, 0, 0), 
                            ARGB(0xff, 0, 0xc0, 0), ARGB(0xff, 0, 0, 0xc0));
    /* ʂ̃AXyNgݒ */
	/* corrects the screen aspect ratio */
    njSetAspect( 1.0F, 1.0F );

    /* eLXg̃J[ݒ */
    /* sets the text color */
    njPrintColor(ARGB(0xff, 0xc0, 0xc0, 0xc0));
}


/**************************************************************************

    njUserMain --- NinjaVXeC֐
                   main
**************************************************************************/

Sint32 njUserMain(void)
{
    NJS_PERIPHERAL *per;

    /* WCXeBbN|[gP̃ytF擾 */
    /* gets the peripheral information */
    per = njGetPeripheral(NJD_PORT_A0);    

    /* {^ꂽǂ`FbN */
    /* checks the start button */
    if (per->press & NJD_DGT_ST) {
        g_bPause = !g_bPause;
    }

	/* |[YԂɂď؂ւ */
	/* pause or not */
	if (!g_bPause) {
		ActivePhase();
	} else {
		PausePhase();
	}

    /* fobO\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(23, 6), "Ninja Sample SHADOW");

    return NJD_USER_CONTINUE;
}


/**************************************************************************

    ActivePhase --- ANeBu֐
                    active (not pause) routine
**************************************************************************/

void ActivePhase(void)
{
    static Angle theta = NJM_DEG_ANG(0.0F);

    /* }gNX̃NA */
    /* clears the matrix stack */
    njClearMatrix();

    /* ړ */
    /* translate */
    njTranslate(NULL, 0.0F, 0.0F, -20.0F);

    /* ] */
    /* rotate */
    theta += NJM_DEG_ANG(0.1F);
    njRotateY(NULL, theta);

    /* ̏ */
    /* light routine */
    LightPhase();

    /* nʂ̕` */
    /* draws plane */
    DrawPlane(10.0F);

    /* IuWFNg̕` */
	/* draws object */
    ObjectPhase();

    /* ȅ */
    /* shadow routine */
    ShadowPhase();
}


/**************************************************************************

    PausePhase --- |[Y֐
                   pause routine
**************************************************************************/

void PausePhase(void)
{
    NJS_POINT2COL pc;
    NJS_POINT2    p[4];
    NJS_COLOR     col[4];
        
    /* Z[o[pXvCg̐ݒ */
    /* sets polygon for screen saver */
    pc.p   = p;
    pc.col = col;
    pc.tex = NULL;
    pc.num = 4;
    
    p[0].x = 0;
    p[0].y = 0;
    col[0].color = 0xff000000;

    p[1].x = 0;
    p[1].y = 480;
    col[1].color = 0xff000000;

    p[2].x = 640;
    p[2].y = 480;
    col[2].color = 0xff000000;

    p[3].x = 640;
    p[3].y = 0;
    col[3].color = 0xff000000;

    /* Z[o[p|S̕` */
    /* draws the screen saver */
    njDrawPolygon2D(&pc, 4, -2.0F, NJD_FILL);

	/* |[Ŷm点 */
	/* prints 'PAUSE' string */
    njPrintC(NJM_LOCATION(38, 30), "PAUSE");
}


/**************************************************************************

    DrawPlane --- ʕ`֐
                  draws plane
**************************************************************************/

void DrawPlane(float size)
{
    NJS_POINT3COL p3col;
    NJS_POINT3    point[4];
    NJS_COLOR     color[4];

    /* _f[^̍쐬 */
    /* sets the vertex data */
    p3col.p   = point;
    p3col.col = color;
    p3col.num = 4;
    
    point[0].x = -size;
    point[0].y =  g_GroundHeight;
    point[0].z = -size;
    color[0].argb.a = 0xff;
    color[0].argb.r = 0x88;
    color[0].argb.g = 0xff;
    color[0].argb.b = 0x88;

    point[1].x = -size;
    point[1].y =  g_GroundHeight;
    point[1].z =  size;
    color[1].argb.a = 0xff;
    color[1].argb.r = 0x88;
    color[1].argb.g = 0xff;
    color[1].argb.b = 0x88;

    point[2].x =  size;
    point[2].y =  g_GroundHeight;
    point[2].z =  size;
    color[2].argb.a = 0xff;
    color[2].argb.r = 0x88;
    color[2].argb.g = 0xff;
    color[2].argb.b = 0x88;

    point[3].x =  size;
    point[3].y =  g_GroundHeight;
    point[3].z = -size;
    color[3].argb.a = 0xff;
    color[3].argb.r = 0x88;
    color[3].argb.g = 0xff;
    color[3].argb.b = 0x88;

    /* ʂ̕` */
    /* draws the plane */
    njPushMatrix(NULL);
        njDrawPolygon3D(&p3col, 4, NJD_FILL);
    njPopMatrix(1);
}


/**************************************************************************

    LightPhase --- ֐
                   light source routine
**************************************************************************/

void LightPhase(void)
{
    NJS_POINT3COL p3c;
    NJS_POINT3    p[50];
    NJS_COLOR     col[50];
    NJS_ARGB      argb;
    int i, j;

    /* ̈ړ */
    /* moves the light source */
    g_LightTheta += NJM_DEG_ANG(1.0F);

    /* ̒nɂW̎Zo */
    /* calculate the coordinates (x,y,z) of the light source */
    g_LightPos.x = g_LightR * njCos(g_LightPhai) * njCos(g_LightTheta);
    g_LightPos.y = g_LightR * njSin(g_LightPhai);
    g_LightPos.z = g_LightR * njCos(g_LightPhai) * njSin(g_LightTheta);
    g_LightDir.x = -g_LightPos.x;
    g_LightDir.y = -g_LightPos.y;
    g_LightDir.z = -g_LightPos.z;

    /* ̈ʒuݒ */
    /* set the light postion and direction */
    njSetLightLocation(&g_Light, g_LightPos.x, g_LightPos.y, g_LightPos.z);
    njSetLightDirection(&g_Light, g_LightDir.x, g_LightDir.y, g_LightDir.z);

    /* }eA̐ݒ */
    /* sets model materials */
    argb.a = 1.0F;
    argb.r = 1.0F;
    argb.g = 1.0F;
    argb.b = 0.0F;
    njSetConstantMaterial( &argb );

    /* ̕` */
    /* draws the arrow that is light direction */
    njPushMatrix(NULL);
        njTranslate(NULL, g_LightPos.x, g_LightPos.y, g_LightPos.z);
        njRotateY(NULL, NJM_DEG_ANG(90.0F) - g_LightTheta);
        njRotateX(NULL, g_LightPhai + NJM_DEG_ANG(180.0F));
        njScale(NULL, 0.1F, 0.1F, 0.1F);
        njDrawObject(object_arrow);
    njPopMatrix(1);

    /* ̐ݒ */
    /* sets the ray of ilght */
    p3c.num = 50;
    p3c.p = p;
    p3c.col = col;

    for (j = 0; j < 5; j++) {
        for (i = 0; i < 5; i++) {
            p3c.p[i * 2 + j * 10].x = g_LightPos.x - 4.0F + i * 2;
            p3c.p[i * 2 + j * 10].y = g_LightPos.y;
            p3c.p[i * 2 + j * 10].z = g_LightPos.z - 4.0F + j * 2;
            p3c.col[i * 2 + j * 10].argb.a = 0xff;
            p3c.col[i * 2 + j * 10].argb.r = 0xff;
            p3c.col[i * 2 + j * 10].argb.g = 0xff;
            p3c.col[i * 2 + j * 10].argb.b = 0xff;
            p3c.p[i * 2 + j * 10 + 1].x = -g_LightPos.x - 4.0F + i * 2;
            p3c.p[i * 2 + j * 10 + 1].y = -g_LightPos.y;
            p3c.p[i * 2 + j * 10 + 1].z = -g_LightPos.z - 4.0F + j * 2;
            p3c.col[i * 2 + j * 10 + 1].argb.a = 0xff;
            p3c.col[i * 2 + j * 10 + 1].argb.r = 0xff;
            p3c.col[i * 2 + j * 10 + 1].argb.g = 0xff;
            p3c.col[i * 2 + j * 10 + 1].argb.b = 0xff;
        }
    }

    /* ̕` */
    /* draws the ray of light */
    njDrawLine3D(&p3c, 25, NJD_DRAW_NORMAL);
}


/**************************************************************************

    ShadowPhase --- e֐

**************************************************************************/

void ShadowPhase(void)
{
    NJS_ARGB  argb;
    NJS_MATRIX shadowMat;
    float dot;

    /* ԏ̑SĂ̓_𕽖ʏɎˉe}gNX̎Zo */
    /* ԏ̓_ S(sx, sy, dz) ʂxNg d = (dx, dy, dz) 
       ̒ L ƕ ax+by+cz+d = 0 Ƃ̌_ S'(s'x, s'y, s'z)߂
       āA̎ _S  _S' ւ̕ϊs}gNX
       ߂B̃}gNX͋ԏ̔Cӂ̓_𕽖 ax+by+cz+d = 0
       ֒ L ɂʑ}gNXł̂ŁAړĨ}gNX
       łBȉł͌vZʂɂƂÂ}gNX𐶐*/

    /* calculate the matrix that project all points to the plane */

    dot = 1.0F * g_LightDir.y / g_LightR;

    shadowMat[M00] = g_LightDir.y / g_LightR / dot;
    shadowMat[M10] = -g_LightDir.x / g_LightR / dot;
    shadowMat[M20] = 0.0F;
    shadowMat[M30] = (g_GroundHeight + 0.1F) * g_LightDir.x / g_LightR / dot;

    shadowMat[M01] = 0.0F;
    shadowMat[M11] = 0.0F;
    shadowMat[M21] = 0.0F;
    shadowMat[M31] = (g_GroundHeight + 0.1F) * g_LightDir.y / g_LightR / dot;

    shadowMat[M02] = 0.0F;
    shadowMat[M12] = -g_LightDir.z / g_LightR / dot;
    shadowMat[M22] = g_LightDir.y / g_LightR / dot;
    shadowMat[M32] = (g_GroundHeight + 0.1F) * g_LightDir.z / g_LightR / dot;

    shadowMat[M03] = 0.0F;
    shadowMat[M13] = 0.0F;
    shadowMat[M23] = 0.0F;
    shadowMat[M33] = 1.0F;

    /* }eA̐ݒ */
    /* sets model materials */
    argb.a = -0.5F;
    argb.r = 0.0F;
    argb.g = 0.0F;
    argb.b = 0.0F;
    njSetConstantMaterial( &argb );

    /* ` */
    /* draws object */
    njPushMatrix(NULL);
        njMultiMatrix(NULL, &shadowMat);
        njDrawObject(object_logo);
    njPopMatrix(1);
}


/**************************************************************************

    ObjectPhase --- IuWFNg֐
                    draw object
**************************************************************************/

void ObjectPhase(void)
{
    NJS_ARGB argb;

    /* }eA̐ݒ */
    /* sets model materials */
    argb.a = 1.0F;
    argb.r = 0.5F;
    argb.g = 1.0F;
    argb.b = 0.5F;
    njSetConstantMaterial( &argb );

    /* IuWFNg̕` */
    /* draws object */
    njPushMatrix(NULL);
        njDrawObject(object_logo);
    njPopMatrix(1);
}


/**************************************************************************

    njUserExit --- NinjaVXeI֐
                   exit
**************************************************************************/

void njUserExit(void)
{
	njExitPrint();

    /* ̊J */
	/* frees the memory area */
    syFree(g_Matrix);
    syFree(g_VBuf);

    /* VXeI֐ */
    /* exits the system    */
    sbExitSystem();
}
