/************************************************************
  Ninja Sample
  COPYRIGHT (C) SEGA ENTERPRISES, LTD.

  ^Cg F ʕ
              separate screen
  ҏW F 98-08-27 Jn
              99-04-05 appends the English comments
************************************************************/

#include <shinobi.h>
#include <njdef.h>

void main(void)
{
	njSetBorderColor( 0x00000000 );
    njUserInit();
    while (njUserMain() == NJD_USER_CONTINUE) {
        njWaitVSync();
    }
    njUserExit();
}

#define MAX_MATRIX_DEPTH 10
#define MAX_VERTEXES 4000

#define SPECULAR_CYCLE 500
#define SPECULAR_CHANGE_SPEED NJM_DEG_ANG(1.0F)

#define SCREEN_SAVE_TIME 30

extern NJS_CNK_OBJECT object__sphere3_sphere3[];

static NJS_MATRIX *Matrix;
static NJS_VERTEX_BUF *VertexBuffer;

static NJS_LIGHT light;
static NJS_VIEW View;

static NJS_POINT2 p1[2];
static NJS_POINT2 p2[2];
static NJS_POINT2 p3[2];
static NJS_POINT2 p4[2];

static NJS_SCREEN scr1;
static NJS_SCREEN scr2;
static NJS_SCREEN scr3;
static NJS_SCREEN scr4;

/*----------------------------------------------------------------------
           
           initialize
----------------------------------------------------------------------*/
void njUserInit(void)
{
    /* VXȅ */
    /* Initializes the Dreamcast H/W and readies the SEGA library */
    sbInitSystem(NJD_RESOLUTION_640x480_NTSCNI, NJD_FRAMEBUFFER_MODE_RGB565, 1);

    /* ʂ̃AXyNgݒ */
	/* corrects the screen aspect ratio */
    njSetAspect( 1.0F, 0.91F );

    /* 蓖 */
    /* allocates the memory */
    Matrix = syMalloc(sizeof(NJS_MATRIX) * MAX_MATRIX_DEPTH);
    VertexBuffer   = syMalloc(sizeof(NJS_VERTEX_BUF) * MAX_VERTEXES);

    /* wiF̐ݒ */
	/* sets background color */
    njSetBackColor(ARGB(0, 192, 0, 0), ARGB(0, 0, 192, 0), ARGB(0, 0, 0, 192));

    /* _i[obt@̊m */
    /* allocates the buffers for registration of vertex data */
    njInitVertexBuffer(MAX_VERTEXES * 64 / 4, 0, MAX_VERTEXES * 64 / 4, 0, 0);
	njInitPrint( NULL, 0, NJD_TEXFMT_ARGB_1555 );

    /* }gNXX^bN̏ */
  	/* initializes the matrix stack */
    njInitMatrix(Matrix, MAX_MATRIX_DEPTH, 1);

    /* 3DVXȅ */
    /* initializes 3D system  */ 
    njInit3D(VertexBuffer, MAX_VERTEXES);
    njControl3D( NJD_CONTROL_3D_CNK_CONSTANT_ATTR );

    /* r[̏ */
    /* initializes the user-defined view */
    njInitView(&View);

    /* r[̐ݒ */
	/* specifies the current view */
    njSetView(&View);

	/* Cg̐ݒ */
    /* initializes the light source */
	njCreateLight(&light, NJD_SPEC_DIR);

	/* F̐ݒ */
    /* sets the text color */
	njPrintColor(0xFFC0C0C0);

	/* \ʒu̒ */
	/* adjusts the display position */
	njAdjustDisplay(0,8);

	p1[0].x = 0;	/* 0x32 = 0 */
	p1[0].y = 0;	/* 0x32 = 0 */
	p1[1].x = 9;	/* 9x32 = 288 */
	p1[1].y = 6;	/* 6x32 = 192 */

	p2[0].x = 10;	/* 10x32 = 320 */
	p2[0].y = 0;	/* 0x32 = 0 */
	p2[1].x = 19;	/* 19x32 = 608 */
	p2[1].y = 6;	/* 6x32 = 192 */

	p3[0].x = 0;	/* 0x32 = 0 */
	p3[0].y = 7;	/* 7x32 = 224 */
	p3[1].x = 9;	/* 9x32 = 288 */
	p3[1].y = 13;	/* 13x32 = 416 */

	p4[0].x = 10;	/* 10x32 = 320 */
	p4[0].y = 7;	/* 7x32 = 224 */
	p4[1].x = 19;	/* 19x32 = 608 */
	p4[1].y = 13;	/* 13x32 = 416 */

	scr1.dist = 250.f;
	scr1.w = 320.f;
	scr1.h = 240.f;
	scr1.cx = 160.f;
	scr1.cy = 120.f;

	scr2.dist = 250.f;
	scr2.w = 320.f;
	scr2.h = 240.f;
	scr2.cx = 480.f;;
	scr2.cy = 120.f;

	scr3.dist = 250.f;
	scr3.w = 320.f;
	scr3.h = 240.f;
	scr3.cx = 160.f;;
	scr3.cy = 360.f;

	scr4.dist = 250.f;
	scr4.w = 320.f;
	scr4.h = 240.f;
	scr4.cx = 480.f;;
	scr4.cy = 360.f;
}

/*----------------------------------------------------------------------
         C
         main
----------------------------------------------------------------------*/
Sint32 njUserMain(void)
{
    static Angle rotateAng = NJM_DEG_ANG(0.0F);/* ]pxpϐ */
                                               /* rotation angle */
    /* XN[W̕ύX */
    /* sets the screen */
	njSetScreen(&scr1);
	
	/* GAP̓ɕ`悵܂ */
	/* draw in the area 1 */
	njUserClipping(NJD_CLIP_INSIDE, p1);

    /* VF[fBO̐ݒ */
    /* sets the shading mode */
    njSetConstantAttr( 0xFFFFFFFF, NJD_FST_FL);

    /* Tv̕\ */
    /* prints the sample program name */
    njPrintC(NJM_LOCATION(4, 4), "Ninja Sample TILECLIP");

    /* }gbNXX^bÑNA */
    /* clears the matrix stack */
    njClearMatrix();

    /* sړ */
    /* translates */
    njTranslate(NULL, 0.F, 0.0F, -25.0F);

    /* X,Y,Z ̏Ԃɉ] */
    /* rotates x,y,z */
    njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);

    /* IuWFNg̕` */
    /* draws the object */
    njCnkDrawObject(object__sphere3_sphere3 );




	njSetScreen(&scr2);
	njUserClipping(NJD_CLIP_INSIDE, p2);
    njSetConstantAttr( ~NJD_FST_FL, 0);
    njClearMatrix();
    njTranslate(NULL, 0.0F, 0.0F, -25.0F);
    njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);
    njCnkDrawObject(object__sphere3_sphere3 );



	njSetScreen(&scr3);
	njUserClipping(NJD_CLIP_INSIDE, p3);
    njSetConstantAttr( ~NJD_FST_FL, 0);
    njClearMatrix();
    njTranslate(NULL, 0.0F, 0.0F, -25.0F);
    njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);
    njCnkDrawObject(object__sphere3_sphere3 );



	njSetScreen(&scr4);
	njUserClipping(NJD_CLIP_INSIDE, p4);
    njSetConstantAttr( 0xFFFFFFFF, NJD_FST_FL);
    njClearMatrix();
    njTranslate(NULL, 0.0F, 0.0F, -25.0F);
    njRotateXYZ(NULL, rotateAng, rotateAng, rotateAng);
    njCnkDrawObject(object__sphere3_sphere3 );

	/* NbsO𒆎~܂ */
	/* stop user clipping */
	njUserClipping(NJD_CLIP_DISABLE, p4);

    rotateAng += SPECULAR_CHANGE_SPEED;

    return NJD_USER_CONTINUE;
}

/*----------------------------------------------------------------------
         I
         exit
----------------------------------------------------------------------*/
void njUserExit(void)
{
    /* ̊J */
	/* frees the memory area */
    syFree(Matrix);
    syFree(VertexBuffer);

    /* VXȅI */
    /* exits the system    */
    njExitPrint();
    sbExitSystem();
}
